

const DIRECTIONS = new Set(['normal', 'reverse', 'alternate', 'alternate-reverse'])
const PLAY_STATES = new Set(['running', 'paused'])
const FILL_MODES = new Set(['none', 'forwards', 'backwards', 'both'])
const ITERATION_COUNTS = new Set(['infinite'])
const TIMINGS = new Set([
  'linear',
  'ease',
  'ease-in',
  'ease-out',
  'ease-in-out',
  'step-start',
  'step-end',
])
const TIMING_FNS22 = ['cubic-bezier', 'steps']

const COMMA = /\,(?![^(]*\))/g // Comma separator that is not located between brackets. E.g.: `cubiz-bezier(a, b, c)` these don't count.
const SPACE = /\ +(?![^(]*\))/g // Similar to the one above, but with spaces instead.
const TIME = /^(-?[\d.]+m?s)$/
const DIGIT = /^(\d+)$/

function asValue(
    modifier,
    lookup = {},
    { validate = () => true, transform = (v) => v } = {}
) {
    let value = lookup[modifier]

    if (value !== undefined) {
        return value
    }

    if (modifier[0] !== '[' || modifier[modifier.length - 1] !== ']') {
        return undefined
    }

    value = modifier.slice(1, -1)

    if (!validate(value)) {
        return undefined
    }

    // add spaces around operators inside calc() that do not follow an operator or (
    return transform(value).replace(
        /(-?\d*\.?\d(?!\b-.+[,)](?![^+\-/*])\D)(?:%|[a-z]+)?|\))([+\-/*])/g,
        '$1 $2 '
    )
}

function asList(modifier, lookup = {}) {
    return asValue(modifier, lookup, {
        transform: (value) => {
            return postcss.list
                .comma(value)
                .map((v) => v.replace(/,/g, ', '))
                .join(' ')
        },
    })
}

function asColor(modifier, lookup = {}, tailwindConfig = {}) {
    if (lookup[modifier] !== undefined) {
        return lookup[modifier]
    }

    let [color, alpha] = splitAlpha(modifier)

    if (lookup[color] !== undefined) {
        if (isArbitraryValue(alpha)) {
            return withAlphaValue(lookup[color], alpha.slice(1, -1))
        }

        if (tailwindConfig.theme?.opacity?.[alpha] === undefined) {
            return undefined
        }

        return withAlphaValue(lookup[color], tailwindConfig.theme.opacity[alpha])
    }

    return asValue(modifier, lookup, {
        validate: isColor,
    })
}

function asAngle(modifier, lookup = {}) {
    return asUnit(modifier, ['deg', 'grad', 'rad', 'turn'], lookup)
}

function asLength(modifier, lookup = {}) {
    return asUnit(
        modifier,
        [
            'cm',
            'mm',
            'Q',
            'in',
            'pc',
            'pt',
            'px',
            'em',
            'ex',
            'ch',
            'rem',
            'lh',
            'vw',
            'vh',
            'vmin',
            'vmax',
            '%',
        ],
        lookup
    )
}

function asLookupValue(modifier, lookup = {}) {
    return lookup[modifier]
}

function escapeCommas(className) {
    return className.replace(/\\,/g, '\\2c ')
}
  

function asClass(name) {
    return escapeCommas(`.${escapeClassName(name)}`)
  }

function escapeClassName(className) {
    return className;
// const node = parser.className()
// node.value = className
// return escapeCommas(get(node, 'raws.value', node.value))
}
  
function nameClass(classPrefix, key) {
    if (key === 'DEFAULT') {
      return asClass(classPrefix)
    }
  
    if (key === '-') {
      return asClass(`-${classPrefix}`)
    }
  
    if (key.startsWith('-')) {
      return asClass(`-${classPrefix}${key}`)
    }
  
    return asClass(`${classPrefix}-${key}`)
  }


let wrap = (obj) => {
    return {
        sortedUniq: () => wrap([...new Set(obj)]),
        value: () => obj,
        map: (f) => wrap(obj.map(f)),
        sortBy: (f) => {
            console.log(obj);
            console.log(f);
            return wrap([...obj].sort(f));
        },
    }; 
}

const spreadableSymbol = Symbol.isConcatSpreadable

function isObjectLike(value) {
    return typeof value === 'object' && value !== null
}
  
const toString = Object.prototype.toString

function getTag(value) {
    if (value == null) {
      return value === undefined ? '[object Undefined]' : '[object Null]'
    }
    return toString.call(value)
  }
  

function isArguments(value) {
    return isObjectLike(value) && getTag(value) == '[object Arguments]'
}

function isFlattenable(value) {
  return Array.isArray(value) || isArguments(value) ||
    !!(value && value[spreadableSymbol])
}

function map(array, iteratee) {
    let index = -1
    const length = array == null ? 0 : array.length
    const result = new Array(length)
  
    while (++index < length) {
      result[index] = iteratee(array[index], index, array)
    }
    return result
}

function baseFlatten(array, depth, predicate, isStrict, result) {
    predicate || (predicate = isFlattenable)
    result || (result = [])
  
    if (array == null) {
      return result
    }
  
    for (const value of array) {
      if (depth > 0 && predicate(value)) {
        if (depth > 1) {
          // Recursively flatten arrays (susceptible to call stack limits).
          baseFlatten(value, depth - 1, predicate, isStrict, result)
        } else {
          result.push(...value)
        }
      } else if (!isStrict) {
        result[result.length] = value
      }
    }
    return result
  }

  function arrayEach(array, iteratee) {
    let index = -1
    const length = array.length
  
    while (++index < length) {
      if (iteratee(array[index], index, array) === false) {
        break
      }
    }
    return array
  }

function negateValue (value) {
    switch (value) {
        case "0px": return "0px";
    }
    return "-" + value

    // try {
    //   return reduceCalc(`calc(${value} * -1)`)
    // } catch (e) {
    //   return "-" + value
    // }
  }
  

  function baseFor(object, iteratee, keysFunc) {
    const iterable = Object(object)
    const props = keysFunc(object)
    let { length } = props
    let index = -1
  
    while (length--) {
      const key = props[++index]
      if (iteratee(iterable[key], key, iterable) === false) {
        break
      }
    }
    return object
  }

  function baseForOwn(object, iteratee) {
    return object && baseFor(object, iteratee, keys)
  }

  function baseEach(collection, iteratee) {
    if (collection == null) {
      return collection
    }
    if (!isArrayLike(collection)) {
      return baseForOwn(collection, iteratee)
    }
    const length = collection.length
    const iterable = Object(collection)
    let index = -1
  
    while (++index < length) {
      if (iteratee(iterable[index], index, iterable) === false) {
        break
      }
    }
    return collection
  }

function forEach(collection, iteratee) {
const func = Array.isArray(collection) ? arrayEach : baseEach
return func(collection, iteratee)
}

let _ = (function () { 
    function main(obj) {
        return wrap(obj);
    }
    main.isString = (value) => { const type = typeof value; return type === 'string' || (type === 'object' && value != null && !Array.isArray(value) && getTag(value) == '[object String]')};
    main.isObject = (value) => {
        const type = typeof value
        return value != null && (type === 'object' || type === 'function')
    };
    main.isFunction = (value) => typeof value === 'function';
    main.flatMap = (collection, iteratee) =>  baseFlatten(map(collection, iteratee), 1);
    main.each = forEach; 
    main.isPlainObject = isPlainObject;
    main.fromPairs = (arr) => arr.reduce((prev, current) => {
        prev[current[0]] = current[1]
        return prev;
    }, {});
    main.map = (arr, func) => Array.isArray(arr) ? [] : Object.keys(arr).map((key1) => func(arr[key1], key1));
    main.find = (arr, func) => arr.find(i => func(i));
    return main;
})();


let fixKey = (key) => {
    switch (key) {
        case "whiteSpace": return "white-space";
        case "borderWidth": return "border-width";
        case "zIndex": return "z-index";
        case "gridColumnEnd": return "grid-column-end";
        case "gridColumnStart": return "grid-column-start";
        case "gridRowStart": return "grid-row-start";
        case "gridRowEnd": return "grid-row-end";
        case "gridRow": return "grid-row";
        case "maxHeight": return "max-height";
        case "minHeight": return "min-height";
        case "minWidth": return "min-width";
        case "maxWidth": return "max-width";
        case "transformOrigin": return "transform-origin";
        case "gridColumn": return "grid-column";
        case "listStyleType": return "list-style-type";
        case "gridAutoColumns": return "grid-auto-columns";
        case "gridAutoFlow": return "grid-auto-flow";
        case "gridTemplateColumns": return "grid-template-columns";
        case "gridTemplateRows": return "grid-template-rows";
        case "gridAutoRows": return "grid-auto-rows";
        case "columnGap": return "column-gap";
        case "rowGap": return "row-gap";
        case "fontWeight": return "font-weight";
        case "lineHeight": return "line-height";
        case "letterSpacing": return "letter-spacing";
        case "fontFamily": return "font-family";
        case "transitionDelay": return "transition-delay";
        case "transitionTimingFunction": return "transition-timing-function";
        case "transitionDuration": return "transition-duration";
        case "animationTimingFunction": return "animation-timing-function";
        case "xxxxx": return "xxxxx";
        case "xxxxx": return "xxxxx";
    }
    return key;
}

// let fixClassName = (key) => {
//     return key[0] + key.slice(1).split("/").join('\\/').split(".").join('\\.');
// }

let fixClassName = (key) => {
    let r = /[^\s]\./m;
    let match = null;
    while(match = key.match(r)) {
        let newChar = match[0].replace(".", "{SPACE}");
        key = key.replace(match[0], newChar);
    }
    return key.split("/").join('\\/').split("{SPACE}").join('\\.')
}

function hexToInt(value) {
    let result = parseInt(value, 16);
    if (isNaN(result)) return '0';
    return result;
}

function hexToRgba(varName, value) {
    if (value=="#fff") return `rgba(255, 255, 255, var(--tw-${varName}-opacity))`
    if (value=="#000") return `rgba(0, 0, 0, var(--tw-${varName}-opacity))`
    return `rgba(${hexToInt(value.slice(1,3))}, ${hexToInt(value.slice(3,5))}, ${hexToInt(value.slice(5,7))}, var(--tw-${varName}-opacity))`
}

function convertColor(className, key, value, props) {

    let varName = "error";

    if (props && props.plugin) {
        varName = props.plugin;
        switch (varName) {
          case "backgroundColor": varName = "bg"; break;
          case "divideColor": varName = "divide"; break;
          case "borderColor": varName = "border"; break;
          case "textColor": varName = "text"; break;
          case "ringColor": varName = "ring"; break;
          case "placeholderColor": varName = "placeholder"; break;
        }
    }
    if (value && value.slice(0,1)=="#") {
        return `  --tw-${varName}-opacity: 1;\n  ${key}: ${hexToRgba(varName, value)};`;
    }
    return `  ${key}: ${value};`;
}

function valueToString(className, key, value, props) {
    switch (key) {
        case 'color': return convertColor(className, key, value, props);
        case 'border-color': return convertColor(className, key, value, props);
        case 'background-color': return convertColor(className, key, value, props);
        case '--tw-ring-color': return convertColor(className, key, value, props);
        case 'user-select': return `  -webkit-user-select: ${value};\n          ${key}: ${value};`;
        case 'appearance': return `  -webkit-appearance: ${value};\n          ${key}: ${value};`;
        case 'box-decoration-break': return `  -webkit-box-decoration-break: ${value};\n          ${key}: ${value};`;
        case 'backdrop-filter': return `  -webkit-backdrop-filter: ${value};\n          ${key}: ${value};`;
        case 'background-clip': {
            if (value=="text")
            return `  -webkit-background-clip: ${value};\n          ${key}: ${value};`;
        }
        
    }
    return `  ${key}: ${value};`;
}

function _generateComplex(className, rules, props) {
  let css = "";

  let pairs = Object.entries(rules);
  let complex = pairs.filter(([key, value]) => typeof value == "object");
  let simple = pairs.filter(([key, value]) => typeof value != "object");

css = css + complex.map(([key, value]) => _generateComplex(key, value, props)).join("");

if (simple.length > 0) {

  css = css + (
`${fixClassName(className)} {
${Object.entries(rules)
  .map(([key, value])=> ([fixKey(key), value]))
  .map(([key, value])=> valueToString(className, key, value, props))
  .join("\n")}
}

`)
} else {
  css = '' + className + ' {\n' + css + '}\n';
}

return css;
};


function _generateCss(className, rules, props) {
  if (className.indexOf("keyframes") > -1) {
    return _generateComplex('@' + className.slice(1), rules, props);
  } 

  let css = "";

    let pairs = Object.entries(rules);
    let complex = pairs.filter(([key, value]) => typeof value == "object");
    let simple = pairs.filter(([key, value]) => typeof value != "object");

  css = css + complex.map(([key, value]) => _generateCss(className + "" + key.slice(1), value, props)).join("\n");

  if (simple.length > 0) {

    css = css + (
`${fixClassName(className)} {
${Object.entries(rules)
    .map(([key, value])=> ([fixKey(key), value]))
    .map(([key, value])=> valueToString(className, key, value, props))
    .join("\n")}
}

`)
  }
  
  return css;
};

function generateExtra(layerSpec) {

    let css = "";
    let key = !layerSpec.key ? layerSpec.key :  layerSpec.key + "\\:"
    css = `.${key}container {
        width: 100%;
    }

    @media (min-width: 640px) {
        .${key}container {
          max-width: 640px;
        }
      }
    
      @media (min-width: 768px) {
        .${key}container {
          max-width: 768px;
        }
      }
    
      @media (min-width: 1024px) {
        .${key}container {
          max-width: 1024px;
        }
      }
    
      @media (min-width: 1280px) {
        .${key}container {
          max-width: 1280px;
        }
      }
    
      @media (min-width: 1536px) {
        .${key}container {
          max-width: 1536px;
        }
      }
    `;
    return css;
}

function generateCss(layerIndex, arr) {
    let css = arr
    .map(wrappedRules => Object.entries(wrappedRules.rules).map(([className, rules])=> _generateCss(className, rules, wrappedRules.props)))
    .flatMap(i=>i)
    .join("");

    if (layerIndex > 0) {
        let layerSpec = layerSpecification[layerIndex];
        let extra = generateExtra(layerSpec);
        return `@media (min-width: ${layerSpec.size}px) {\n${extra}\n` + css + "}\n";
    } else {
        let layerSpec = layerSpecification[layerIndex];
        let extra = generateExtra(layerSpec);
        return `${extra}` + css;
    }
}


/* process plugins */

function addUtilities(utilities, options) {
    const defaultOptions = {
      variants: [],
      respectPrefix: true,
      respectImportant: true,
    }

    options = Array.isArray(options)
      ? Object.assign({}, defaultOptions, { variants: options })
      : _.defaults(options, defaultOptions)

    // const styles = postcss.root({ nodes: parseStyles(utilities) })

    // styles.walkRules((rule) => {
    //   if (options.respectPrefix && !isKeyframeRule(rule)) {
    //     rule.selector = applyConfiguredPrefix(rule.selector)
    //   }

    //   if (options.respectImportant && config.important) {
    //     rule.__tailwind = {
    //       ...rule.__tailwind,
    //       important: config.important,
    //     }
    //   }
    // })

    pluginUtilities.push(
      wrapWithLayer(wrapWithVariants(styles.nodes, options.variants), 'utilities')
    )
  }

function fixVaiantName(str) {
    if (str == "responsive") return "";
    return str;
}

const objectProto = Object.prototype
const hasOwnProperty = objectProto.hasOwnProperty

function eq(value, other) {
    return value === other || (value !== value && other !== other)
}

function defaults(object, ...sources) {
    object = Object(object)
    sources.forEach((source) => {
      if (source != null) {
        source = Object(source)
        for (const key in source) {
          const value = object[key]
          if (value === undefined ||
              (eq(value, objectProto[key]) && !hasOwnProperty.call(object, key))) {
            object[key] = source[key]
          }
        }
      }
    })
    return object
}
function wrapRules(rules, variant, layerInfo, currentPlugin) {
    let prefix = "";
    let slice = 0;
    if (layerInfo.key) {
        prefix = layerInfo.key + "\\:";
        slice = 1;
    }
    let array = Object.entries(rules).filter(([key]) => layerInfo.key ? key.indexOf("keyframes") == -1 : true).map(([key, values]) => {
      let vname = fixVaiantName(variant);
      if (vname) {
          if (vname=="group-hover") {
            return [`.group:hover .${prefix}group-hover\\:${key.slice(1)}`, values];
          } else {
            return [`.${prefix}${vname}\\:${key.slice(1)}:${vname}`, values];
          }
      } else if (key[0]=="*") {
        return layerInfo.key ? [`skip`, {}] : [`${key}`, values];
      } else {
        return [`.${prefix}${key.slice(1)}`, values];
      }
    });
    let processedRules = Object.fromEntries(array);
    return {
      rules: processedRules, 
      props: {
        layer: layerInfo,
        plugin: currentPlugin
      }
    };
  }
  
function wrapWithVariants4(arrayOfRules, variants, layerInfo, currentPlugin) {
    try {
        arrayOfRules = Array.isArray(arrayOfRules) ? arrayOfRules : [arrayOfRules];
        if (!layerInfo) {
            arrayOfRules.filter(r => r);
        }
        let result = variants.filter(v => v!="dark").map(variant => {
            return arrayOfRules.map(rules => wrapRules(rules, variant, layerInfo, currentPlugin))
        });
        return result.flatMap(f=>f);
    } catch (e) {
        console.error(e);
    }
}

function wrapWithVariants2(rules, variants, layerInfo, currentPlugin) {
    return wrapWithVariants4(rules, variants, layerInfo, currentPlugin);
}

function wrapWithLayers(layers, func) {
    layers.map((layer, index) => func(layer, index));
}

let layerSpecification = [{key:"", size:0},{key:"sm", size:640},{key:"md", size:768},{key:"lg", size:1024},{key:"xl", size:1280},{key:"\\32xl", size:1536}];  

function addUtilities2(utilities, options, currentPlugin) {
    const defaultOptions = { variants: [], respectPrefix: true, respectImportant: true };
    options = Array.isArray(options) ? Object.assign({}, defaultOptions, { variants: options }) : defaults(options, defaultOptions);

    if (!options.variants || !Array.isArray(options.variants) || options.variants.length == 0) {
        options.variants = ["responsive"];
    }


    wrapWithLayers(layerSpecification, (layerInfo, index) => {
        if (tailwindResult.length <= index) {
            tailwindResult.push([]);
        }
        if (!((currentPlugin=="caretColor" || currentPlugin=="content") && layerInfo.key != "")) {
            let withVariants = wrapWithVariants2(utilities, options.variants, layerInfo, currentPlugin);
            withVariants.map(wrappedRules => {
                tailwindResult[index].push(wrappedRules)
            });
        } else {
            // console.log("skippi");
        }
    } );

    // console.log(JSON.stringify(withVariants, null, 2));
    // console.log(JSON.stringify(utilities, null, 2));
    // console.log(JSON.stringify(options.variants, null, 2));
    // process.exit();
}

  const charCodeOfDot = '.'.charCodeAt(0)
  const reEscapeChar = /\\(\\)?/g
  const rePropName = RegExp(
    // Match anything that isn't a dot or bracket.
    '[^.[\\]]+' + '|' +
    // Or match property names within brackets.
    '\\[(?:' +
      // Match a non-string expression.
      '([^"\'][^[]*)' + '|' +
      // Or match strings (supports escaping characters).
      '(["\'])((?:(?!\\2)[^\\\\]|\\\\.)*?)\\2' +
    ')\\]'+ '|' +
    // Or match "" as the space between consecutive dots or empty brackets.
    '(?=(?:\\.|\\[\\])(?:\\.|\\[\\]|$))'
    , 'g')
  
  const stringToPath = memoizeCapped((string) => {
    const result = []
    if (string.charCodeAt(0) === charCodeOfDot) {
      result.push('')
    }
    string.replace(rePropName, (match, expression, quote, subString) => {
      let key = match
      if (quote) {
        key = subString.replace(reEscapeChar, '$1')
      }
      else if (expression) {
        key = expression.trim()
      }
      result.push(key)
    })
    return result
  })

function toPath(value) {
    if (Array.isArray(value)) {
      return map(value, toKey)
    }
    return isSymbol(value) ? [value] : copyArray(stringToPath(value))
}  


// const resolveConfigObjects = require('./lib/util/resolveConfig').default
// const getAllConfigs = require('./lib/util/getAllConfigs').default

// function resolveConfig(...configs) {
//   const [, ...defaultConfigs] = getAllConfigs(configs[0])
//   return resolveConfigObjects([...configs, ...defaultConfigs])
// }


function move(items, item, befores) {
    let lowestBefore = -1
  
    for (let before of befores) {
      let index = items.indexOf(before)
      if (index >= 0 && (index < lowestBefore || lowestBefore === -1)) {
        lowestBefore = index
      }
    }
  
    if (items.indexOf(item) === -1 || lowestBefore === -1) {
      return items
    }
  
    items = [...items]
    let fromIndex = items.indexOf(item)
    let toIndex = lowestBefore
    items.splice(fromIndex, 1)
    items.splice(toIndex, 0, item)
    return items
  }

 function getPlugins ({ corePlugins: corePluginConfig }) {
    let pluginOrder = Object.keys(plugins)
  
    pluginOrder = configurePlugins(corePluginConfig, pluginOrder)
    pluginOrder = move(pluginOrder, 'transform', ['translate', 'rotate', 'skew', 'scale'])
    pluginOrder = move(pluginOrder, 'filter', [
      'blur',
      'brightness',
      'contrast',
      'dropShadow',
      'grayscale',
      'hueRotate',
      'invert',
      'saturate',
      'sepia',
    ])
    pluginOrder = move(pluginOrder, 'backdropFilter', [
      'backdropBlur',
      'backdropBrightness',
      'backdropContrast',
      'backdropGrayscale',
      'backdropHueRotate',
      'backdropInvert',
      'backdropOpacity',
      'backdropSaturate',
      'backdropSepia',
    ])
  
    return pluginOrder;
 }

let myPluginOrder = 
 [
    // "preflight",
    "container",
    "accessibility",
    "pointerEvents",
    "visibility",
    "position",
    "inset",
    "isolation",
    "zIndex",
    "order",
    "gridColumn",
    "gridColumnStart",
    "gridColumnEnd",
    "gridRow",
    "gridRowStart",
    "gridRowEnd",
    "float",
    "clear",
    "margin",
    "boxSizing",
    "display",
    "height",
    "maxHeight",
    "minHeight",
    "width",
    "minWidth",
    "maxWidth",
    "flex",
    "flexShrink",
    "flexGrow",
    "tableLayout",
    "borderCollapse",
    "transformOrigin",
    "transform",
    "translate",
    "rotate",
    "skew",
    "scale",
    "animation",
    "cursor",
    "userSelect",
    "resize",
    "listStylePosition",
    "listStyleType",
    "appearance",
    "gridAutoColumns",
    "gridAutoFlow",
    "gridAutoRows",
    "gridTemplateColumns",
    "gridTemplateRows",
    "flexDirection",
    "flexWrap",
    "placeContent",
    "placeItems",
    "alignContent",
    "alignItems",
    "justifyContent",
    "justifyItems",
    "gap",
    "space",
    "divideWidth",
    "divideStyle",
    "divideColor",
    "divideOpacity",
    "placeSelf",
    "alignSelf",
    "justifySelf",
    "overflow",
    "overscrollBehavior",
    "textOverflow",
    "whitespace",
    "wordBreak",
    "borderRadius",
    "borderWidth",
    "borderStyle",
    "borderColor",
    "borderOpacity",
    "backgroundColor",
    "backgroundOpacity",
    "backgroundImage",
    "gradientColorStops",
    "boxDecorationBreak",
    "backgroundSize",
    "backgroundAttachment",
    "backgroundClip",
    "backgroundPosition",
    "backgroundRepeat",
    "backgroundOrigin",
    "fill",
    "stroke",
    "strokeWidth",
    "objectFit",
    "objectPosition",
    "padding",
    "textAlign",
    "verticalAlign",
    "fontFamily",
    "fontSize",
    "fontWeight",
    "textTransform",
    "fontStyle",
    "fontVariantNumeric",
    "lineHeight",
    "letterSpacing",
    "textColor",
    "textOpacity",
    "textDecoration",
    "fontSmoothing",
    "placeholderColor",
    "placeholderOpacity",
    "caretColor",
    "opacity",
    "backgroundBlendMode",
    "mixBlendMode",
    "boxShadow",
    "outline",
    "ringWidth",
    "ringColor",
    "ringOpacity",
    "ringOffsetWidth",
    "ringOffsetColor",
    "filter",
    "blur",
    "brightness",
    "contrast",
    "dropShadow",
    "grayscale",
    "hueRotate",
    "invert",
    "saturate",
    "sepia",
    "backdropFilter",
    "backdropBlur",
    "backdropBrightness",
    "backdropContrast",
    "backdropGrayscale",
    "backdropHueRotate",
    "backdropInvert",
    "backdropOpacity",
    "backdropSaturate",
    "backdropSepia",
    "transitionProperty",
    "transitionDelay",
    "transitionDuration",
    "transitionTimingFunction",
    "content"
  ]

//   .rgb().array()
  function createColor(value) {

    function hexToInt(v) {
        let result = parseInt(v, 16);
        if (isNaN(result)) {
            console.error("bad color: " + value, v);
            return 0;
        }
        return result;
    }

    return {
        rgb: () => {
            return {
                array: () => {
                    if (value=="transparent") return [0,0,0,0];
                    if (value=="currentColor") return [255,255,255,0];
                    if (value=="#fff") return [255,255,255,1];
                    if (value=="#000") return [0,0,0,1];
                    return [hexToInt(value.slice(1,3)),hexToInt(value.slice(3,5)),hexToInt(value.slice(5,7)),1];
                    return 
                }                
            }
        }
    }
  }
  
const preflightCss = `/**
 * Manually forked from SUIT CSS Base: https://github.com/suitcss/base
 * A thin layer on top of normalize.css that provides a starting point more
 * suitable for web applications.
 */

/**
 * Removes the default spacing and border for appropriate elements.
 */

blockquote,
dl,
dd,
h1,
h2,
h3,
h4,
h5,
h6,
hr,
figure,
p,
pre {
  margin: 0;
}

button {
  background-color: transparent;
  background-image: none;
}

fieldset {
  margin: 0;
  padding: 0;
}

ol,
ul {
  list-style: none;
  margin: 0;
  padding: 0;
}

/**
 * Tailwind custom reset styles
 */

/**
 * 1. Use the user's configured \`sans\` font-family (with Tailwind's default
 *    sans-serif font stack as a fallback) as a sane default.
 * 2. Use Tailwind's default "normal" line-height so the user isn't forced
 *    to override it to ensure consistency even when using the default theme.
 */

html {
  font-family: theme('fontFamily.sans', ui-sans-serif, system-ui, -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji"); /* 1 */
  line-height: 1.5; /* 2 */
}


/**
 * Inherit font-family and line-height from \`html\` so users can set them as
 * a class directly on the \`html\` element.
 */

body {
  font-family: inherit;
  line-height: inherit;
}

/**
 * 1. Prevent padding and border from affecting element width.
 *
 *    We used to set this in the html element and inherit from
 *    the parent element for everything else. This caused issues
 *    in shadow-dom-enhanced elements like <details> where the content
 *    is wrapped by a div with box-sizing set to \`content-box\`.
 *
 *    https://github.com/mozdevs/cssremedy/issues/4
 *
 *
 * 2. Allow adding a border to an element by just adding a border-width.
 *
 *    By default, the way the browser specifies that an element should have no
 *    border is by setting it's border-style to \`none\` in the user-agent
 *    stylesheet.
 *
 *    In order to easily add borders to elements by just setting the \`border-width\`
 *    property, we change the default border-style for all elements to \`solid\`, and
 *    use border-width to hide them instead. This way our \`border\` utilities only
 *    need to set the \`border-width\` property instead of the entire \`border\`
 *    shorthand, making our border utilities much more straightforward to compose.
 *
 *    https://github.com/tailwindcss/tailwindcss/pull/116
 */

*,
::before,
::after {
  box-sizing: border-box; /* 1 */
  border-width: 0; /* 2 */
  border-style: solid; /* 2 */
  border-color: currentColor; /* 2 */
}

/*
 * Ensure horizontal rules are visible by default
 */

hr {
  border-top-width: 1px;
}

/**
 * Undo the \`border-style: none\` reset that Normalize applies to images so that
 * our \`border-{width}\` utilities have the expected effect.
 *
 * The Normalize reset is unnecessary for us since we default the border-width
 * to 0 on all elements.
 *
 * https://github.com/tailwindcss/tailwindcss/issues/362
 */

img {
  border-style: solid;
}

textarea {
  resize: vertical;
}

input::placeholder,
textarea::placeholder {
  opacity: 1;
  color: #9ca3af;
}

button,
[role="button"] {
  cursor: pointer;
}

/**
 * Override legacy focus reset from Normalize with modern Firefox focus styles.
 *
 * This is actually an improvement over the new defaults in Firefox in our testing,
 * as it triggers the better focus styles even for links, which still use a dotted
 * outline in Firefox by default.
 */
 
:-moz-focusring {
	outline: auto;
}

table {
  border-collapse: collapse;
}

h1,
h2,
h3,
h4,
h5,
h6 {
  font-size: inherit;
  font-weight: inherit;
}

/**
 * Reset links to optimize for opt-in styling instead of
 * opt-out.
 */

a {
  color: inherit;
  text-decoration: inherit;
}

/**
 * Reset form element properties that are easy to forget to
 * style explicitly so you don't inadvertently introduce
 * styles that deviate from your design system. These styles
 * supplement a partial reset that is already applied by
 * normalize.css.
 */

button,
input,
optgroup,
select,
textarea {
  padding: 0;
  line-height: inherit;
  color: inherit;
}

/**
 * Use the configured 'mono' font family for elements that
 * are expected to be rendered with a monospace font, falling
 * back to the system monospace stack if there is no configured
 * 'mono' font family.
 */

pre,
code,
kbd,
samp {
  font-family: ui-monospace, SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace;
}

/**
 * 1. Make replaced elements \`display: block\` by default as that's
 *    the behavior you want almost all of the time. Inspired by
 *    CSS Remedy, with \`svg\` added as well.
 *
 *    https://github.com/mozdevs/cssremedy/issues/14
 * 
 * 2. Add \`vertical-align: middle\` to align replaced elements more
 *    sensibly by default when overriding \`display\` by adding a
 *    utility like \`inline\`.
 *
 *    This can trigger a poorly considered linting error in some
 *    tools but is included by design.
 * 
 *    https://github.com/jensimmons/cssremedy/issues/14#issuecomment-634934210
 */

img,
svg,
video,
canvas,
audio,
iframe,
embed,
object {
  display: block; /* 1 */
  vertical-align: middle; /* 2 */
}

/**
 * Constrain images and videos to the parent width and preserve
 * their intrinsic aspect ratio.
 *
 * https://github.com/mozdevs/cssremedy/issues/14
 */

img,
video {
  max-width: 100%;
  height: auto;
}

/**
 * Ensure the default browser behavior of the \`hidden\` attribute.
 */

[hidden] {
  display: none;
}
`;

const modernNormalizeCss = `/*! modern-normalize v1.1.0 | MIT License | https://github.com/sindresorhus/modern-normalize */

/*
Document
========
*/

/**
Use a better box model (opinionated).
*/

*,
::before,
::after {
	box-sizing: border-box;
}

/**
Use a more readable tab size (opinionated).
*/

html {
	-moz-tab-size: 4;
	tab-size: 4;
}

/**
1. Correct the line height in all browsers.
2. Prevent adjustments of font size after orientation changes in iOS.
*/

html {
	line-height: 1.15; /* 1 */
	-webkit-text-size-adjust: 100%; /* 2 */
}

/*
Sections
========
*/

/**
Remove the margin in all browsers.
*/

body {
	margin: 0;
}

/**
Improve consistency of default fonts in all browsers. (https://github.com/sindresorhus/modern-normalize/issues/3)
*/

body {
	font-family:
		system-ui,
		-apple-system, /* Firefox supports this but not yet \`system-ui\` */
		'Segoe UI',
		Roboto,
		Helvetica,
		Arial,
		sans-serif,
		'Apple Color Emoji',
		'Segoe UI Emoji';
}

/*
Grouping content
================
*/

/**
1. Add the correct height in Firefox.
2. Correct the inheritance of border color in Firefox. (https://bugzilla.mozilla.org/show_bug.cgi?id=190655)
*/

hr {
	height: 0; /* 1 */
	color: inherit; /* 2 */
}

/*
Text-level semantics
====================
*/

/**
Add the correct text decoration in Chrome, Edge, and Safari.
*/

abbr[title] {
	text-decoration: underline dotted;
}

/**
Add the correct font weight in Edge and Safari.
*/

b,
strong {
	font-weight: bolder;
}

/**
1. Improve consistency of default fonts in all browsers. (https://github.com/sindresorhus/modern-normalize/issues/3)
2. Correct the odd 'em' font sizing in all browsers.
*/

code,
kbd,
samp,
pre {
	font-family:
		ui-monospace,
		SFMono-Regular,
		Consolas,
		'Liberation Mono',
		Menlo,
		monospace; /* 1 */
	font-size: 1em; /* 2 */
}

/**
Add the correct font size in all browsers.
*/

small {
	font-size: 80%;
}

/**
Prevent 'sub' and 'sup' elements from affecting the line height in all browsers.
*/

sub,
sup {
	font-size: 75%;
	line-height: 0;
	position: relative;
	vertical-align: baseline;
}

sub {
	bottom: -0.25em;
}

sup {
	top: -0.5em;
}

/*
Tabular data
============
*/

/**
1. Remove text indentation from table contents in Chrome and Safari. (https://bugs.chromium.org/p/chromium/issues/detail?id=999088, https://bugs.webkit.org/show_bug.cgi?id=201297)
2. Correct table border color inheritance in all Chrome and Safari. (https://bugs.chromium.org/p/chromium/issues/detail?id=935729, https://bugs.webkit.org/show_bug.cgi?id=195016)
*/

table {
	text-indent: 0; /* 1 */
	border-color: inherit; /* 2 */
}

/*
Forms
=====
*/

/**
1. Change the font styles in all browsers.
2. Remove the margin in Firefox and Safari.
*/

button,
input,
optgroup,
select,
textarea {
	font-family: inherit; /* 1 */
	font-size: 100%; /* 1 */
	line-height: 1.15; /* 1 */
	margin: 0; /* 2 */
}

/**
Remove the inheritance of text transform in Edge and Firefox.
1. Remove the inheritance of text transform in Firefox.
*/

button,
select { /* 1 */
	text-transform: none;
}

/**
Correct the inability to style clickable types in iOS and Safari.
*/

button,
[type='button'],
[type='reset'],
[type='submit'] {
	-webkit-appearance: button;
}

/**
Remove the inner border and padding in Firefox.
*/

::-moz-focus-inner {
	border-style: none;
	padding: 0;
}

/**
Restore the focus styles unset by the previous rule.
*/

:-moz-focusring {
	outline: 1px dotted ButtonText;
}

/**
Remove the additional ':invalid' styles in Firefox.
See: https://github.com/mozilla/gecko-dev/blob/2f9eacd9d3d995c937b4251a5557d95d494c9be1/layout/style/res/forms.css#L728-L737
*/

:-moz-ui-invalid {
	box-shadow: none;
}

/**
Remove the padding so developers are not caught out when they zero out 'fieldset' elements in all browsers.
*/

legend {
	padding: 0;
}

/**
Add the correct vertical alignment in Chrome and Firefox.
*/

progress {
	vertical-align: baseline;
}

/**
Correct the cursor style of increment and decrement buttons in Safari.
*/

::-webkit-inner-spin-button,
::-webkit-outer-spin-button {
	height: auto;
}

/**
1. Correct the odd appearance in Chrome and Safari.
2. Correct the outline style in Safari.
*/

[type='search'] {
	-webkit-appearance: textfield; /* 1 */
	outline-offset: -2px; /* 2 */
}

/**
Remove the inner padding in Chrome and Safari on macOS.
*/

::-webkit-search-decoration {
	-webkit-appearance: none;
}

/**
1. Correct the inability to style clickable types in iOS and Safari.
2. Change font properties to 'inherit' in Safari.
*/

::-webkit-file-upload-button {
	-webkit-appearance: button; /* 1 */
	font: inherit; /* 2 */
}

/*
Interactive
===========
*/

/*
Add the correct display in Chrome and Safari.
*/

summary {
	display: list-item;
}
`;



function  toColorValue(maybeFunction) {
  return _.isFunction(maybeFunction) ? maybeFunction({}) : maybeFunction
}
function  isPlainObject(value) {
  if (Object.prototype.toString.call(value) !== '[object Object]') {
    return false
  }

  const prototype = Object.getPrototypeOf(value)
  return prototype === null || prototype === Object.prototype
}



function hasAlpha(color) {
  return (
    color.startsWith('rgba(') ||
    color.startsWith('hsla(') ||
    (color.startsWith('#') && color.length === 9) ||
    (color.startsWith('#') && color.length === 5)
  )
}

function toRgba(color) {
  const [r, g, b, a] = createColor(color).rgb().array()

  return [r, g, b, a === undefined && hasAlpha(color) ? 1 : a]
}

function toHsla(color) {
  const [h, s, l, a] = createColor(color).hsl().array()

  return [h, `${s}%`, `${l}%`, a === undefined && hasAlpha(color) ? 1 : a]
}

function withAlphaValue(color, alphaValue, defaultValue) {
  if (_.isFunction(color)) {
    return color({ opacityValue: alphaValue })
  }

  try {
    const isHSL = color.startsWith('hsl')
    const [i, j, k] = isHSL ? toHsla(color) : toRgba(color)
    return `${isHSL ? 'hsla' : 'rgba'}(${i}, ${j}, ${k}, ${alphaValue})`
  } catch (e) {
    console.error(e);
    return defaultValue
  }
}

function  withAlphaVariable({ color, property, variable }) {
  if (_.isFunction(color)) {
    return {
      [variable]: '1',
      [property]: color({ opacityVariable: variable, opacityValue: `var(${variable})` }),
    }
  }

  try {
    const isHSL = color.startsWith('hsl')

    const [i, j, k, a] = isHSL ? toHsla(color) : toRgba(color)

    if (a !== undefined) {
      return {
        [property]: color,
      }
    }

    return {
      [variable]: '1',
      [property]: `${isHSL ? 'hsla' : 'rgba'}(${i}, ${j}, ${k}, var(${variable}))`,
    }
  } catch (error) {
    return {
      [property]: color,
    }
  }
}
const flattenColorPalette = (colors) =>
  Object.assign(
    {},
    ...Object.entries(colors).flatMap(([color, values]) =>
      typeof values == 'object'
        ? Object.entries(flattenColorPalette(values)).map(([number, hex]) => ({
            [color + (number === 'DEFAULT' ? '' : `-${number}`)]: hex,
          }))
        : [{ [`${color}`]: values }]
    )
  )







const TIMING_FNS = ['cubic-bezier', 'steps']






function  parseAnimationValue(input) {
  let animations = input.split(COMMA)
  let result = animations.map((animation) => {
    let result = {}
    let parts = animation.trim().split(SPACE)
    let seen = new Set()

    for (let part of parts) {
      if (!seen.has('DIRECTIONS') && DIRECTIONS.has(part)) {
        result.direction = part
        seen.add('DIRECTIONS')
      } else if (!seen.has('PLAY_STATES') && PLAY_STATES.has(part)) {
        result.playState = part
        seen.add('PLAY_STATES')
      } else if (!seen.has('FILL_MODES') && FILL_MODES.has(part)) {
        result.fillMode = part
        seen.add('FILL_MODES')
      } else if (
        !seen.has('ITERATION_COUNTS') &&
        (ITERATION_COUNTS.has(part) || DIGIT.test(part))
      ) {
        result.iterationCount = part
        seen.add('ITERATION_COUNTS')
      } else if (!seen.has('TIMING_FUNCTION') && TIMINGS.has(part)) {
        result.timingFunction = part
        seen.add('TIMING_FUNCTION')
      } else if (!seen.has('TIMING_FUNCTION') && TIMING_FNS.some((f) => part.startsWith(`${f}(`))) {
        result.timingFunction = part
        seen.add('TIMING_FUNCTION')
      } else if (!seen.has('DURATION') && TIME.test(part)) {
        result.duration = part
        seen.add('DURATION')
      } else if (!seen.has('DELAY') && TIME.test(part)) {
        result.delay = part
        seen.add('DELAY')
      } else result.name = part
    }

    return result
  })

  return animations.length > 1 ? result : result[0]
}



let asMap = new Map([
  [asValue, 'any'],
  [asList, 'list'],
  [asColor, 'color'],
  [asAngle, 'angle'],
  [asLength, 'length'],
  [asLookupValue, 'lookup'],
])

function  createUtilityPlugin(
  themeKey,
  utilityVariations = [[themeKey, [themeKey]]],
  { filterDefault = false, resolveArbitraryValue = asValue } = {}
) {
  let transformValue = transformThemeValue(themeKey)
  return function ({ matchUtilities, variants, theme }) {
    for (let utilityVariation of utilityVariations) {
      let group = Array.isArray(utilityVariation[0]) ? utilityVariation : [utilityVariation]

      matchUtilities(
        group.reduce((obj, [classPrefix, properties]) => {
          return Object.assign(obj, {
            [classPrefix]: (value) => {
              return properties.reduce((obj, name) => {
                if (Array.isArray(name)) {
                  return Object.assign(obj, { [name[0]]: name[1] })
                }
                return Object.assign(obj, { [name]: transformValue(value) })
              }, {})
            },
          })
        }, {}),
        {
          values: filterDefault
            ? Object.fromEntries(
                Object.entries(theme(themeKey)).filter(([modifier]) => modifier !== 'DEFAULT')
              )
            : theme(themeKey),
          variants: variants(themeKey),
          type: asMap.get(resolveArbitraryValue) ?? 'any',
        }
      )
    }
  }
}
function  transformThemeValue(themeSection) {
  if (['fontSize', 'outline'].includes(themeSection)) {
    return (value) => (Array.isArray(value) ? value[0] : value)
  }

  if (
    [
      'fontFamily',
      'boxShadow',
      'transitionProperty',
      'transitionDuration',
      'transitionDelay',
      'transitionTimingFunction',
      'backgroundImage',
      'backgroundSize',
      'backgroundColor',
      'cursor',
      'animation',
    ].includes(themeSection)
  ) {
    return (value) => (Array.isArray(value) ? value.join(', ') : value)
  }

  if (themeSection === 'colors') {
    return (value) => (typeof value === 'function' ? value({}) : value)
  }

  return (value) => value
}
let configurePlugins = function (pluginConfig, plugins) {
  if (pluginConfig === undefined) {
    return plugins
  }

  const pluginNames = Array.isArray(pluginConfig)
    ? pluginConfig
    : [
        ...new Set(
          plugins
            .filter((pluginName) => {
              return pluginConfig !== false && pluginConfig[pluginName] !== false
            })
            .concat(
              Object.keys(pluginConfig).filter((pluginName) => {
                return pluginConfig[pluginName] !== false
              })
            )
        ),
      ]

  return pluginNames
}
let warned = false

let colors =  {
  black: '#000',
  white: '#fff',
  rose: {
    50: '#fff1f2',
    100: '#ffe4e6',
    200: '#fecdd3',
    300: '#fda4af',
    400: '#fb7185',
    500: '#f43f5e',
    600: '#e11d48',
    700: '#be123c',
    800: '#9f1239',
    900: '#881337',
  },
  pink: {
    50: '#fdf2f8',
    100: '#fce7f3',
    200: '#fbcfe8',
    300: '#f9a8d4',
    400: '#f472b6',
    500: '#ec4899',
    600: '#db2777',
    700: '#be185d',
    800: '#9d174d',
    900: '#831843',
  },
  fuchsia: {
    50: '#fdf4ff',
    100: '#fae8ff',
    200: '#f5d0fe',
    300: '#f0abfc',
    400: '#e879f9',
    500: '#d946ef',
    600: '#c026d3',
    700: '#a21caf',
    800: '#86198f',
    900: '#701a75',
  },
  purple: {
    50: '#faf5ff',
    100: '#f3e8ff',
    200: '#e9d5ff',
    300: '#d8b4fe',
    400: '#c084fc',
    500: '#a855f7',
    600: '#9333ea',
    700: '#7e22ce',
    800: '#6b21a8',
    900: '#581c87',
  },
  violet: {
    50: '#f5f3ff',
    100: '#ede9fe',
    200: '#ddd6fe',
    300: '#c4b5fd',
    400: '#a78bfa',
    500: '#8b5cf6',
    600: '#7c3aed',
    700: '#6d28d9',
    800: '#5b21b6',
    900: '#4c1d95',
  },
  indigo: {
    50: '#eef2ff',
    100: '#e0e7ff',
    200: '#c7d2fe',
    300: '#a5b4fc',
    400: '#818cf8',
    500: '#6366f1',
    600: '#4f46e5',
    700: '#4338ca',
    800: '#3730a3',
    900: '#312e81',
  },
  blue: {
    50: '#eff6ff',
    100: '#dbeafe',
    200: '#bfdbfe',
    300: '#93c5fd',
    400: '#60a5fa',
    500: '#3b82f6',
    600: '#2563eb',
    700: '#1d4ed8',
    800: '#1e40af',
    900: '#1e3a8a',
  },
  get lightBlue() {
    if (!warned) {
      console.log('warn - As of Tailwind CSS v2.2, `lightBlue` has been renamed to `sky`.')
      console.log('warn - Please update your color palette to eliminate this warning.')
      warned = true
    }
    return {
      50: '#f0f9ff',
      100: '#e0f2fe',
      200: '#bae6fd',
      300: '#7dd3fc',
      400: '#38bdf8',
      500: '#0ea5e9',
      600: '#0284c7',
      700: '#0369a1',
      800: '#075985',
      900: '#0c4a6e',
    }
  },
  sky: {
    50: '#f0f9ff',
    100: '#e0f2fe',
    200: '#bae6fd',
    300: '#7dd3fc',
    400: '#38bdf8',
    500: '#0ea5e9',
    600: '#0284c7',
    700: '#0369a1',
    800: '#075985',
    900: '#0c4a6e',
  },
  cyan: {
    50: '#ecfeff',
    100: '#cffafe',
    200: '#a5f3fc',
    300: '#67e8f9',
    400: '#22d3ee',
    500: '#06b6d4',
    600: '#0891b2',
    700: '#0e7490',
    800: '#155e75',
    900: '#164e63',
  },
  teal: {
    50: '#f0fdfa',
    100: '#ccfbf1',
    200: '#99f6e4',
    300: '#5eead4',
    400: '#2dd4bf',
    500: '#14b8a6',
    600: '#0d9488',
    700: '#0f766e',
    800: '#115e59',
    900: '#134e4a',
  },
  emerald: {
    50: '#ecfdf5',
    100: '#d1fae5',
    200: '#a7f3d0',
    300: '#6ee7b7',
    400: '#34d399',
    500: '#10b981',
    600: '#059669',
    700: '#047857',
    800: '#065f46',
    900: '#064e3b',
  },
  green: {
    50: '#f0fdf4',
    100: '#dcfce7',
    200: '#bbf7d0',
    300: '#86efac',
    400: '#4ade80',
    500: '#22c55e',
    600: '#16a34a',
    700: '#15803d',
    800: '#166534',
    900: '#14532d',
  },
  lime: {
    50: '#f7fee7',
    100: '#ecfccb',
    200: '#d9f99d',
    300: '#bef264',
    400: '#a3e635',
    500: '#84cc16',
    600: '#65a30d',
    700: '#4d7c0f',
    800: '#3f6212',
    900: '#365314',
  },
  yellow: {
    50: '#fefce8',
    100: '#fef9c3',
    200: '#fef08a',
    300: '#fde047',
    400: '#facc15',
    500: '#eab308',
    600: '#ca8a04',
    700: '#a16207',
    800: '#854d0e',
    900: '#713f12',
  },
  amber: {
    50: '#fffbeb',
    100: '#fef3c7',
    200: '#fde68a',
    300: '#fcd34d',
    400: '#fbbf24',
    500: '#f59e0b',
    600: '#d97706',
    700: '#b45309',
    800: '#92400e',
    900: '#78350f',
  },
  orange: {
    50: '#fff7ed',
    100: '#ffedd5',
    200: '#fed7aa',
    300: '#fdba74',
    400: '#fb923c',
    500: '#f97316',
    600: '#ea580c',
    700: '#c2410c',
    800: '#9a3412',
    900: '#7c2d12',
  },
  red: {
    50: '#fef2f2',
    100: '#fee2e2',
    200: '#fecaca',
    300: '#fca5a5',
    400: '#f87171',
    500: '#ef4444',
    600: '#dc2626',
    700: '#b91c1c',
    800: '#991b1b',
    900: '#7f1d1d',
  },
  warmGray: {
    50: '#fafaf9',
    100: '#f5f5f4',
    200: '#e7e5e4',
    300: '#d6d3d1',
    400: '#a8a29e',
    500: '#78716c',
    600: '#57534e',
    700: '#44403c',
    800: '#292524',
    900: '#1c1917',
  },
  trueGray: {
    50: '#fafafa',
    100: '#f5f5f5',
    200: '#e5e5e5',
    300: '#d4d4d4',
    400: '#a3a3a3',
    500: '#737373',
    600: '#525252',
    700: '#404040',
    800: '#262626',
    900: '#171717',
  },
  gray: {
    50: '#fafafa',
    100: '#f4f4f5',
    200: '#e4e4e7',
    300: '#d4d4d8',
    400: '#a1a1aa',
    500: '#71717a',
    600: '#52525b',
    700: '#3f3f46',
    800: '#27272a',
    900: '#18181b',
  },
  coolGray: {
    50: '#f9fafb',
    100: '#f3f4f6',
    200: '#e5e7eb',
    300: '#d1d5db',
    400: '#9ca3af',
    500: '#6b7280',
    600: '#4b5563',
    700: '#374151',
    800: '#1f2937',
    900: '#111827',
  },
  blueGray: {
    50: '#f8fafc',
    100: '#f1f5f9',
    200: '#e2e8f0',
    300: '#cbd5e1',
    400: '#94a3b8',
    500: '#64748b',
    600: '#475569',
    700: '#334155',
    800: '#1e293b',
    900: '#0f172a',
  },
}


let defaultConfig =  {
  purge: [],
  presets: [],
  darkMode: false, // or 'media' or 'class'
  theme: {
    screens: {
      sm: '640px',
      md: '768px',
      lg: '1024px',
      xl: '1280px',
      '2xl': '1536px',
    },
    colors: {
      transparent: 'transparent',
      current: 'currentColor',

      black: colors.black,
      white: colors.white,
      gray: colors.coolGray,
      red: colors.red,
      yellow: colors.amber,
      green: colors.emerald,
      blue: colors.blue,
      indigo: colors.indigo,
      purple: colors.violet,
      pink: colors.pink,
    },
    spacing: {
      px: '1px',
      0: '0px',
      0.5: '0.125rem',
      1: '0.25rem',
      1.5: '0.375rem',
      2: '0.5rem',
      2.5: '0.625rem',
      3: '0.75rem',
      3.5: '0.875rem',
      4: '1rem',
      5: '1.25rem',
      6: '1.5rem',
      7: '1.75rem',
      8: '2rem',
      9: '2.25rem',
      10: '2.5rem',
      11: '2.75rem',
      12: '3rem',
      14: '3.5rem',
      16: '4rem',
      20: '5rem',
      24: '6rem',
      28: '7rem',
      32: '8rem',
      36: '9rem',
      40: '10rem',
      44: '11rem',
      48: '12rem',
      52: '13rem',
      56: '14rem',
      60: '15rem',
      64: '16rem',
      72: '18rem',
      80: '20rem',
      96: '24rem',
    },
    animation: {
      none: 'none',
      spin: 'spin 1s linear infinite',
      ping: 'ping 1s cubic-bezier(0, 0, 0.2, 1) infinite',
      pulse: 'pulse 2s cubic-bezier(0.4, 0, 0.6, 1) infinite',
      bounce: 'bounce 1s infinite',
    },
    backdropBlur: (theme) => theme('blur'),
    backdropBrightness: (theme) => theme('brightness'),
    backdropContrast: (theme) => theme('contrast'),
    backdropGrayscale: (theme) => theme('grayscale'),
    backdropHueRotate: (theme) => theme('hueRotate'),
    backdropInvert: (theme) => theme('invert'),
    backdropOpacity: (theme) => theme('opacity'),
    backdropSaturate: (theme) => theme('saturate'),
    backdropSepia: (theme) => theme('sepia'),
    backgroundColor: (theme) => theme('colors'),
    backgroundImage: {
      none: 'none',
      'gradient-to-t': 'linear-gradient(to top, var(--tw-gradient-stops))',
      'gradient-to-tr': 'linear-gradient(to top right, var(--tw-gradient-stops))',
      'gradient-to-r': 'linear-gradient(to right, var(--tw-gradient-stops))',
      'gradient-to-br': 'linear-gradient(to bottom right, var(--tw-gradient-stops))',
      'gradient-to-b': 'linear-gradient(to bottom, var(--tw-gradient-stops))',
      'gradient-to-bl': 'linear-gradient(to bottom left, var(--tw-gradient-stops))',
      'gradient-to-l': 'linear-gradient(to left, var(--tw-gradient-stops))',
      'gradient-to-tl': 'linear-gradient(to top left, var(--tw-gradient-stops))',
    },
    backgroundOpacity: (theme) => theme('opacity'),
    backgroundPosition: {
      bottom: 'bottom',
      center: 'center',
      left: 'left',
      'left-bottom': 'left bottom',
      'left-top': 'left top',
      right: 'right',
      'right-bottom': 'right bottom',
      'right-top': 'right top',
      top: 'top',
    },
    backgroundSize: {
      auto: 'auto',
      cover: 'cover',
      contain: 'contain',
    },
    blur: {
      0: '0',
      none: '0',
      sm: '4px',
      DEFAULT: '8px',
      md: '12px',
      lg: '16px',
      xl: '24px',
      '2xl': '40px',
      '3xl': '64px',
    },
    brightness: {
      0: '0',
      50: '.5',
      75: '.75',
      90: '.9',
      95: '.95',
      100: '1',
      105: '1.05',
      110: '1.1',
      125: '1.25',
      150: '1.5',
      200: '2',
    },
    borderColor: (theme) => ({
      ...theme('colors'),
      DEFAULT: theme('colors.gray.200', 'currentColor'),
    }),
    borderOpacity: (theme) => theme('opacity'),
    borderRadius: {
      none: '0px',
      sm: '0.125rem',
      DEFAULT: '0.25rem',
      md: '0.375rem',
      lg: '0.5rem',
      xl: '0.75rem',
      '2xl': '1rem',
      '3xl': '1.5rem',
      full: '9999px',
    },
    borderWidth: {
      DEFAULT: '1px',
      0: '0px',
      2: '2px',
      4: '4px',
      8: '8px',
    },
    boxShadow: {
      sm: '0 1px 2px 0 rgba(0, 0, 0, 0.05)',
      DEFAULT: '0 1px 3px 0 rgba(0, 0, 0, 0.1), 0 1px 2px 0 rgba(0, 0, 0, 0.06)',
      md: '0 4px 6px -1px rgba(0, 0, 0, 0.1), 0 2px 4px -1px rgba(0, 0, 0, 0.06)',
      lg: '0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05)',
      xl: '0 20px 25px -5px rgba(0, 0, 0, 0.1), 0 10px 10px -5px rgba(0, 0, 0, 0.04)',
      '2xl': '0 25px 50px -12px rgba(0, 0, 0, 0.25)',
      inner: 'inset 0 2px 4px 0 rgba(0, 0, 0, 0.06)',
      none: 'none',
    },
    caretColor: (theme) => theme('colors'),
    contrast: {
      0: '0',
      50: '.5',
      75: '.75',
      100: '1',
      125: '1.25',
      150: '1.5',
      200: '2',
    },
    container: {},
    content: {
      none: 'none',
    },
    cursor: {
      auto: 'auto',
      default: 'default',
      pointer: 'pointer',
      wait: 'wait',
      text: 'text',
      move: 'move',
      help: 'help',
      'not-allowed': 'not-allowed',
    },
    divideColor: (theme) => theme('borderColor'),
    divideOpacity: (theme) => theme('borderOpacity'),
    divideWidth: (theme) => theme('borderWidth'),
    dropShadow: {
      sm: '0 1px 1px rgba(0,0,0,0.05)',
      DEFAULT: ['0 1px 2px rgba(0, 0, 0, 0.1)', '0 1px 1px rgba(0, 0, 0, 0.06)'],
      md: ['0 4px 3px rgba(0, 0, 0, 0.07)', '0 2px 2px rgba(0, 0, 0, 0.06)'],
      lg: ['0 10px 8px rgba(0, 0, 0, 0.04)', '0 4px 3px rgba(0, 0, 0, 0.1)'],
      xl: ['0 20px 13px rgba(0, 0, 0, 0.03)', '0 8px 5px rgba(0, 0, 0, 0.08)'],
      '2xl': '0 25px 25px rgba(0, 0, 0, 0.15)',
      none: '0 0 #0000',
    },
    fill: { current: 'currentColor' },
    grayscale: {
      0: '0',
      DEFAULT: '100%',
    },
    hueRotate: {
      '-180': '-180deg',
      '-90': '-90deg',
      '-60': '-60deg',
      '-30': '-30deg',
      '-15': '-15deg',
      0: '0deg',
      15: '15deg',
      30: '30deg',
      60: '60deg',
      90: '90deg',
      180: '180deg',
    },
    invert: {
      0: '0',
      DEFAULT: '100%',
    },
    flex: {
      1: '1 1 0%',
      auto: '1 1 auto',
      initial: '0 1 auto',
      none: 'none',
    },
    flexGrow: {
      0: '0',
      DEFAULT: '1',
    },
    flexShrink: {
      0: '0',
      DEFAULT: '1',
    },
    fontFamily: {
      sans: [
        'ui-sans-serif',
        'system-ui',
        '-apple-system',
        'BlinkMacSystemFont',
        '"Segoe UI"',
        'Roboto',
        '"Helvetica Neue"',
        'Arial',
        '"Noto Sans"',
        'sans-serif',
        '"Apple Color Emoji"',
        '"Segoe UI Emoji"',
        '"Segoe UI Symbol"',
        '"Noto Color Emoji"',
      ],
      serif: ['ui-serif', 'Georgia', 'Cambria', '"Times New Roman"', 'Times', 'serif'],
      mono: [
        'ui-monospace',
        'SFMono-Regular',
        'Menlo',
        'Monaco',
        'Consolas',
        '"Liberation Mono"',
        '"Courier New"',
        'monospace',
      ],
    },
    fontSize: {
      xs: ['0.75rem', { lineHeight: '1rem' }],
      sm: ['0.875rem', { lineHeight: '1.25rem' }],
      base: ['1rem', { lineHeight: '1.5rem' }],
      lg: ['1.125rem', { lineHeight: '1.75rem' }],
      xl: ['1.25rem', { lineHeight: '1.75rem' }],
      '2xl': ['1.5rem', { lineHeight: '2rem' }],
      '3xl': ['1.875rem', { lineHeight: '2.25rem' }],
      '4xl': ['2.25rem', { lineHeight: '2.5rem' }],
      '5xl': ['3rem', { lineHeight: '1' }],
      '6xl': ['3.75rem', { lineHeight: '1' }],
      '7xl': ['4.5rem', { lineHeight: '1' }],
      '8xl': ['6rem', { lineHeight: '1' }],
      '9xl': ['8rem', { lineHeight: '1' }],
    },
    fontWeight: {
      thin: '100',
      extralight: '200',
      light: '300',
      normal: '400',
      medium: '500',
      semibold: '600',
      bold: '700',
      extrabold: '800',
      black: '900',
    },
    gap: (theme) => theme('spacing'),
    gradientColorStops: (theme) => theme('colors'),
    gridAutoColumns: {
      auto: 'auto',
      min: 'min-content',
      max: 'max-content',
      fr: 'minmax(0, 1fr)',
    },
    gridAutoRows: {
      auto: 'auto',
      min: 'min-content',
      max: 'max-content',
      fr: 'minmax(0, 1fr)',
    },
    gridColumn: {
      auto: 'auto',
      'span-1': 'span 1 / span 1',
      'span-2': 'span 2 / span 2',
      'span-3': 'span 3 / span 3',
      'span-4': 'span 4 / span 4',
      'span-5': 'span 5 / span 5',
      'span-6': 'span 6 / span 6',
      'span-7': 'span 7 / span 7',
      'span-8': 'span 8 / span 8',
      'span-9': 'span 9 / span 9',
      'span-10': 'span 10 / span 10',
      'span-11': 'span 11 / span 11',
      'span-12': 'span 12 / span 12',
      'span-full': '1 / -1',
    },
    gridColumnEnd: {
      auto: 'auto',
      1: '1',
      2: '2',
      3: '3',
      4: '4',
      5: '5',
      6: '6',
      7: '7',
      8: '8',
      9: '9',
      10: '10',
      11: '11',
      12: '12',
      13: '13',
    },
    gridColumnStart: {
      auto: 'auto',
      1: '1',
      2: '2',
      3: '3',
      4: '4',
      5: '5',
      6: '6',
      7: '7',
      8: '8',
      9: '9',
      10: '10',
      11: '11',
      12: '12',
      13: '13',
    },
    gridRow: {
      auto: 'auto',
      'span-1': 'span 1 / span 1',
      'span-2': 'span 2 / span 2',
      'span-3': 'span 3 / span 3',
      'span-4': 'span 4 / span 4',
      'span-5': 'span 5 / span 5',
      'span-6': 'span 6 / span 6',
      'span-full': '1 / -1',
    },
    gridRowStart: {
      auto: 'auto',
      1: '1',
      2: '2',
      3: '3',
      4: '4',
      5: '5',
      6: '6',
      7: '7',
    },
    gridRowEnd: {
      auto: 'auto',
      1: '1',
      2: '2',
      3: '3',
      4: '4',
      5: '5',
      6: '6',
      7: '7',
    },
    gridTemplateColumns: {
      none: 'none',
      1: 'repeat(1, minmax(0, 1fr))',
      2: 'repeat(2, minmax(0, 1fr))',
      3: 'repeat(3, minmax(0, 1fr))',
      4: 'repeat(4, minmax(0, 1fr))',
      5: 'repeat(5, minmax(0, 1fr))',
      6: 'repeat(6, minmax(0, 1fr))',
      7: 'repeat(7, minmax(0, 1fr))',
      8: 'repeat(8, minmax(0, 1fr))',
      9: 'repeat(9, minmax(0, 1fr))',
      10: 'repeat(10, minmax(0, 1fr))',
      11: 'repeat(11, minmax(0, 1fr))',
      12: 'repeat(12, minmax(0, 1fr))',
    },
    gridTemplateRows: {
      none: 'none',
      1: 'repeat(1, minmax(0, 1fr))',
      2: 'repeat(2, minmax(0, 1fr))',
      3: 'repeat(3, minmax(0, 1fr))',
      4: 'repeat(4, minmax(0, 1fr))',
      5: 'repeat(5, minmax(0, 1fr))',
      6: 'repeat(6, minmax(0, 1fr))',
    },
    height: (theme) => ({
      auto: 'auto',
      ...theme('spacing'),
      '1/2': '50%',
      '1/3': '33.333333%',
      '2/3': '66.666667%',
      '1/4': '25%',
      '2/4': '50%',
      '3/4': '75%',
      '1/5': '20%',
      '2/5': '40%',
      '3/5': '60%',
      '4/5': '80%',
      '1/6': '16.666667%',
      '2/6': '33.333333%',
      '3/6': '50%',
      '4/6': '66.666667%',
      '5/6': '83.333333%',
      full: '100%',
      screen: '100vh',
    }),
    inset: (theme, { negative }) => ({
      auto: 'auto',
      ...theme('spacing'),
      ...negative(theme('spacing')),
      '1/2': '50%',
      '1/3': '33.333333%',
      '2/3': '66.666667%',
      '1/4': '25%',
      '2/4': '50%',
      '3/4': '75%',
      full: '100%',
      '-1/2': '-50%',
      '-1/3': '-33.333333%',
      '-2/3': '-66.666667%',
      '-1/4': '-25%',
      '-2/4': '-50%',
      '-3/4': '-75%',
      '-full': '-100%',
    }),
    keyframes: {
      spin: {
        to: {
          transform: 'rotate(360deg)',
        },
      },
      ping: {
        '75%, 100%': {
          transform: 'scale(2)',
          opacity: '0',
        },
      },
      pulse: {
        '50%': {
          opacity: '.5',
        },
      },
      bounce: {
        '0%, 100%': {
          transform: 'translateY(-25%)',
          animationTimingFunction: 'cubic-bezier(0.8,0,1,1)',
        },
        '50%': {
          transform: 'none',
          animationTimingFunction: 'cubic-bezier(0,0,0.2,1)',
        },
      },
    },
    letterSpacing: {
      tighter: '-0.05em',
      tight: '-0.025em',
      normal: '0em',
      wide: '0.025em',
      wider: '0.05em',
      widest: '0.1em',
    },
    lineHeight: {
      none: '1',
      tight: '1.25',
      snug: '1.375',
      normal: '1.5',
      relaxed: '1.625',
      loose: '2',
      3: '.75rem',
      4: '1rem',
      5: '1.25rem',
      6: '1.5rem',
      7: '1.75rem',
      8: '2rem',
      9: '2.25rem',
      10: '2.5rem',
    },
    listStyleType: {
      none: 'none',
      disc: 'disc',
      decimal: 'decimal',
    },
    margin: (theme, { negative }) => ({
      auto: 'auto',
      ...theme('spacing'),
      ...negative(theme('spacing')),
    }),
    maxHeight: (theme) => ({
      ...theme('spacing'),
      full: '100%',
      screen: '100vh',
    }),
    maxWidth: (theme, { breakpoints }) => ({
      none: 'none',
      0: '0rem',
      xs: '20rem',
      sm: '24rem',
      md: '28rem',
      lg: '32rem',
      xl: '36rem',
      '2xl': '42rem',
      '3xl': '48rem',
      '4xl': '56rem',
      '5xl': '64rem',
      '6xl': '72rem',
      '7xl': '80rem',
      full: '100%',
      min: 'min-content',
      max: 'max-content',
      prose: '65ch',
      ...breakpoints(theme('screens')),
    }),
    minHeight: {
      0: '0px',
      full: '100%',
      screen: '100vh',
    },
    minWidth: {
      0: '0px',
      full: '100%',
      min: 'min-content',
      max: 'max-content',
    },
    objectPosition: {
      bottom: 'bottom',
      center: 'center',
      left: 'left',
      'left-bottom': 'left bottom',
      'left-top': 'left top',
      right: 'right',
      'right-bottom': 'right bottom',
      'right-top': 'right top',
      top: 'top',
    },
    opacity: {
      0: '0',
      5: '0.05',
      10: '0.1',
      20: '0.2',
      25: '0.25',
      30: '0.3',
      40: '0.4',
      50: '0.5',
      60: '0.6',
      70: '0.7',
      75: '0.75',
      80: '0.8',
      90: '0.9',
      95: '0.95',
      100: '1',
    },
    order: {
      first: '-9999',
      last: '9999',
      none: '0',
      1: '1',
      2: '2',
      3: '3',
      4: '4',
      5: '5',
      6: '6',
      7: '7',
      8: '8',
      9: '9',
      10: '10',
      11: '11',
      12: '12',
    },
    outline: {
      none: ['2px solid transparent', '2px'],
      white: ['2px dotted white', '2px'],
      black: ['2px dotted black', '2px'],
    },
    padding: (theme) => theme('spacing'),
    placeholderColor: (theme) => theme('colors'),
    placeholderOpacity: (theme) => theme('opacity'),
    ringColor: (theme) => ({
      DEFAULT: theme('colors.blue.500', '#3b82f6'),
      ...theme('colors'),
    }),
    ringOffsetColor: (theme) => theme('colors'),
    ringOffsetWidth: {
      0: '0px',
      1: '1px',
      2: '2px',
      4: '4px',
      8: '8px',
    },
    ringOpacity: (theme) => ({
      DEFAULT: '0.5',
      ...theme('opacity'),
    }),
    ringWidth: {
      DEFAULT: '3px',
      0: '0px',
      1: '1px',
      2: '2px',
      4: '4px',
      8: '8px',
    },
    rotate: {
      '-180': '-180deg',
      '-90': '-90deg',
      '-45': '-45deg',
      '-12': '-12deg',
      '-6': '-6deg',
      '-3': '-3deg',
      '-2': '-2deg',
      '-1': '-1deg',
      0: '0deg',
      1: '1deg',
      2: '2deg',
      3: '3deg',
      6: '6deg',
      12: '12deg',
      45: '45deg',
      90: '90deg',
      180: '180deg',
    },
    saturate: {
      0: '0',
      50: '.5',
      100: '1',
      150: '1.5',
      200: '2',
    },
    scale: {
      0: '0',
      50: '.5',
      75: '.75',
      90: '.9',
      95: '.95',
      100: '1',
      105: '1.05',
      110: '1.1',
      125: '1.25',
      150: '1.5',
    },
    sepia: {
      0: '0',
      DEFAULT: '100%',
    },
    skew: {
      '-12': '-12deg',
      '-6': '-6deg',
      '-3': '-3deg',
      '-2': '-2deg',
      '-1': '-1deg',
      0: '0deg',
      1: '1deg',
      2: '2deg',
      3: '3deg',
      6: '6deg',
      12: '12deg',
    },
    space: (theme, { negative }) => ({
      ...theme('spacing'),
      ...negative(theme('spacing')),
    }),
    stroke: {
      current: 'currentColor',
    },
    strokeWidth: {
      0: '0',
      1: '1',
      2: '2',
    },
    textColor: (theme) => theme('colors'),
    textOpacity: (theme) => theme('opacity'),
    transformOrigin: {
      center: 'center',
      top: 'top',
      'top-right': 'top right',
      right: 'right',
      'bottom-right': 'bottom right',
      bottom: 'bottom',
      'bottom-left': 'bottom left',
      left: 'left',
      'top-left': 'top left',
    },
    transitionDelay: {
      75: '75ms',
      100: '100ms',
      150: '150ms',
      200: '200ms',
      300: '300ms',
      500: '500ms',
      700: '700ms',
      1000: '1000ms',
    },
    transitionDuration: {
      DEFAULT: '150ms',
      75: '75ms',
      100: '100ms',
      150: '150ms',
      200: '200ms',
      300: '300ms',
      500: '500ms',
      700: '700ms',
      1000: '1000ms',
    },
    transitionProperty: {
      none: 'none',
      all: 'all',
      DEFAULT:
        'background-color, border-color, color, fill, stroke, opacity, box-shadow, transform, filter, backdrop-filter',
      colors: 'background-color, border-color, color, fill, stroke',
      opacity: 'opacity',
      shadow: 'box-shadow',
      transform: 'transform',
    },
    transitionTimingFunction: {
      DEFAULT: 'cubic-bezier(0.4, 0, 0.2, 1)',
      linear: 'linear',
      in: 'cubic-bezier(0.4, 0, 1, 1)',
      out: 'cubic-bezier(0, 0, 0.2, 1)',
      'in-out': 'cubic-bezier(0.4, 0, 0.2, 1)',
    },
    translate: (theme, { negative }) => ({
      ...theme('spacing'),
      ...negative(theme('spacing')),
      '1/2': '50%',
      '1/3': '33.333333%',
      '2/3': '66.666667%',
      '1/4': '25%',
      '2/4': '50%',
      '3/4': '75%',
      full: '100%',
      '-1/2': '-50%',
      '-1/3': '-33.333333%',
      '-2/3': '-66.666667%',
      '-1/4': '-25%',
      '-2/4': '-50%',
      '-3/4': '-75%',
      '-full': '-100%',
    }),
    width: (theme) => ({
      auto: 'auto',
      ...theme('spacing'),
      '1/2': '50%',
      '1/3': '33.333333%',
      '2/3': '66.666667%',
      '1/4': '25%',
      '2/4': '50%',
      '3/4': '75%',
      '1/5': '20%',
      '2/5': '40%',
      '3/5': '60%',
      '4/5': '80%',
      '1/6': '16.666667%',
      '2/6': '33.333333%',
      '3/6': '50%',
      '4/6': '66.666667%',
      '5/6': '83.333333%',
      '1/12': '8.333333%',
      '2/12': '16.666667%',
      '3/12': '25%',
      '4/12': '33.333333%',
      '5/12': '41.666667%',
      '6/12': '50%',
      '7/12': '58.333333%',
      '8/12': '66.666667%',
      '9/12': '75%',
      '10/12': '83.333333%',
      '11/12': '91.666667%',
      full: '100%',
      screen: '100vw',
      min: 'min-content',
      max: 'max-content',
    }),
    zIndex: {
      auto: 'auto',
      0: '0',
      10: '10',
      20: '20',
      30: '30',
      40: '40',
      50: '50',
    },
  },
  variantOrder: [
    'first',
    'last',
    'odd',
    'even',
    'visited',
    'checked',
    'empty',
    'read-only',
    'group-hover',
    'group-focus',
    'focus-within',
    'hover',
    'focus',
    'focus-visible',
    'active',
    'disabled',
  ],
  variants: {
    accessibility: ['responsive', 'focus-within', 'focus'],
    alignContent: ['responsive'],
    alignItems: ['responsive'],
    alignSelf: ['responsive'],
    animation: ['responsive'],
    appearance: ['responsive'],
    backdropBlur: ['responsive'],
    backdropBrightness: ['responsive'],
    backdropContrast: ['responsive'],
    backdropFilter: ['responsive'],
    backdropGrayscale: ['responsive'],
    backdropHueRotate: ['responsive'],
    backdropInvert: ['responsive'],
    backdropOpacity: ['responsive'],
    backdropSaturate: ['responsive'],
    backdropSepia: ['responsive'],
    backgroundAttachment: ['responsive'],
    backgroundBlendMode: ['responsive'],
    backgroundClip: ['responsive'],
    backgroundColor: ['responsive', 'dark', 'group-hover', 'focus-within', 'hover', 'focus'],
    backgroundImage: ['responsive'],
    backgroundOpacity: ['responsive', 'dark', 'group-hover', 'focus-within', 'hover', 'focus'],
    backgroundPosition: ['responsive'],
    backgroundRepeat: ['responsive'],
    backgroundSize: ['responsive'],
    backgroundOrigin: ['responsive'],
    blur: ['responsive'],
    borderCollapse: ['responsive'],
    borderColor: ['responsive', 'dark', 'group-hover', 'focus-within', 'hover', 'focus'],
    borderOpacity: ['responsive', 'dark', 'group-hover', 'focus-within', 'hover', 'focus'],
    borderRadius: ['responsive'],
    borderStyle: ['responsive'],
    borderWidth: ['responsive'],
    boxDecorationBreak: ['responsive'],
    boxShadow: ['responsive', 'group-hover', 'focus-within', 'hover', 'focus'],
    boxSizing: ['responsive'],
    brightness: ['responsive'],
    clear: ['responsive'],
    container: ['responsive'],
    contrast: ['responsive'],
    cursor: ['responsive'],
    display: ['responsive'],
    divideColor: ['responsive', 'dark'],
    divideOpacity: ['responsive', 'dark'],
    divideStyle: ['responsive'],
    divideWidth: ['responsive'],
    dropShadow: ['responsive'],
    fill: ['responsive'],
    filter: ['responsive'],
    flex: ['responsive'],
    flexDirection: ['responsive'],
    flexGrow: ['responsive'],
    flexShrink: ['responsive'],
    flexWrap: ['responsive'],
    float: ['responsive'],
    fontFamily: ['responsive'],
    fontSize: ['responsive'],
    fontSmoothing: ['responsive'],
    fontStyle: ['responsive'],
    fontVariantNumeric: ['responsive'],
    fontWeight: ['responsive'],
    gap: ['responsive'],
    gradientColorStops: ['responsive', 'dark', 'hover', 'focus'],
    grayscale: ['responsive'],
    gridAutoColumns: ['responsive'],
    gridAutoFlow: ['responsive'],
    gridAutoRows: ['responsive'],
    gridColumn: ['responsive'],
    gridColumnEnd: ['responsive'],
    gridColumnStart: ['responsive'],
    gridRow: ['responsive'],
    gridRowEnd: ['responsive'],
    gridRowStart: ['responsive'],
    gridTemplateColumns: ['responsive'],
    gridTemplateRows: ['responsive'],
    height: ['responsive'],
    hueRotate: ['responsive'],
    inset: ['responsive'],
    invert: ['responsive'],
    isolation: ['responsive'],
    justifyContent: ['responsive'],
    justifyItems: ['responsive'],
    justifySelf: ['responsive'],
    letterSpacing: ['responsive'],
    lineHeight: ['responsive'],
    listStylePosition: ['responsive'],
    listStyleType: ['responsive'],
    margin: ['responsive'],
    maxHeight: ['responsive'],
    maxWidth: ['responsive'],
    minHeight: ['responsive'],
    minWidth: ['responsive'],
    mixBlendMode: ['responsive'],
    objectFit: ['responsive'],
    objectPosition: ['responsive'],
    opacity: ['responsive', 'group-hover', 'focus-within', 'hover', 'focus'],
    order: ['responsive'],
    outline: ['responsive', 'focus-within', 'focus'],
    overflow: ['responsive'],
    overscrollBehavior: ['responsive'],
    padding: ['responsive'],
    placeContent: ['responsive'],
    placeItems: ['responsive'],
    placeSelf: ['responsive'],
    placeholderColor: ['responsive', 'dark', 'focus'],
    placeholderOpacity: ['responsive', 'dark', 'focus'],
    pointerEvents: ['responsive'],
    position: ['responsive'],
    resize: ['responsive'],
    ringColor: ['responsive', 'dark', 'focus-within', 'focus'],
    ringOffsetColor: ['responsive', 'dark', 'focus-within', 'focus'],
    ringOffsetWidth: ['responsive', 'focus-within', 'focus'],
    ringOpacity: ['responsive', 'dark', 'focus-within', 'focus'],
    ringWidth: ['responsive', 'focus-within', 'focus'],
    rotate: ['responsive', 'hover', 'focus'],
    saturate: ['responsive'],
    scale: ['responsive', 'hover', 'focus'],
    sepia: ['responsive'],
    skew: ['responsive', 'hover', 'focus'],
    space: ['responsive'],
    stroke: ['responsive'],
    strokeWidth: ['responsive'],
    tableLayout: ['responsive'],
    textAlign: ['responsive'],
    textColor: ['responsive', 'dark', 'group-hover', 'focus-within', 'hover', 'focus'],
    textDecoration: ['responsive', 'group-hover', 'focus-within', 'hover', 'focus'],
    textOpacity: ['responsive', 'dark', 'group-hover', 'focus-within', 'hover', 'focus'],
    textOverflow: ['responsive'],
    textTransform: ['responsive'],
    transform: ['responsive'],
    transformOrigin: ['responsive'],
    transitionDelay: ['responsive'],
    transitionDuration: ['responsive'],
    transitionProperty: ['responsive'],
    transitionTimingFunction: ['responsive'],
    translate: ['responsive', 'hover', 'focus'],
    userSelect: ['responsive'],
    verticalAlign: ['responsive'],
    visibility: ['responsive'],
    whitespace: ['responsive'],
    width: ['responsive'],
    wordBreak: ['responsive'],
    zIndex: ['responsive', 'focus-within', 'focus'],
  },
  plugins: [],
}

/* eslint-disable no-shadow */


function extractMinWidths(breakpoints) {
  return _.flatMap(breakpoints, (breakpoints) => {
    if (_.isString(breakpoints)) {
      breakpoints = { min: breakpoints }
    }

    if (!Array.isArray(breakpoints)) {
      breakpoints = [breakpoints]
    }

    return _(breakpoints)
      .filter((breakpoint) => {
        return _.has(breakpoint, 'min') || _.has(breakpoint, 'min-width')
      })
      .map((breakpoint) => {
        return _.get(breakpoint, 'min-width', breakpoint.min)
      })
      .value()
  })
}

function mapMinWidthsToPadding(minWidths, screens, paddings) {
  if (typeof paddings === 'undefined') {
    return []
  }

  if (!_.isObject(paddings)) {
    return [
      {
        screen: 'DEFAULT',
        minWidth: 0,
        padding: paddings,
      },
    ]
  }

  const mapping = []

  if (paddings.DEFAULT) {
    mapping.push({
      screen: 'DEFAULT',
      minWidth: 0,
      padding: paddings.DEFAULT,
    })
  }

  _.each(minWidths, (minWidth) => {
    Object.keys(screens).forEach((screen) => {
      const screenMinWidth = _.isPlainObject(screens[screen])
        ? screens[screen].min || screens[screen]['min-width']
        : screens[screen]

      if (`${screenMinWidth}` === `${minWidth}`) {
        mapping.push({
          screen,
          minWidth,
          padding: paddings[screen],
        })
      }
    })
  })

  return mapping
}

function container() {
  return function ({ addComponents, theme, variants }) {
    const screens = theme('container.screens', theme('screens'))
    const minWidths = extractMinWidths(screens)
    const paddings = mapMinWidthsToPadding(minWidths, screens, theme('container.padding'))

    const generatePaddingFor = (minWidth) => {
      const paddingConfig = _.find(paddings, (padding) => `${padding.minWidth}` === `${minWidth}`)

      if (!paddingConfig) {
        return {}
      }

      return {
        paddingRight: paddingConfig.padding,
        paddingLeft: paddingConfig.padding,
      }
    }

    const atRules = _(minWidths)
      .sortBy((minWidth) => parseInt(minWidth))
      .sortedUniq()
      .map((minWidth) => {
        return {
          [`@media (min-width: ${minWidth})`]: {
            '.container': {
              'max-width': minWidth,
              ...generatePaddingFor(minWidth),
            },
          },
        }
      })
      .value()

    addComponents(
      [
        {
          '.container': Object.assign(
            { width: '100%' },
            theme('container.center', false) ? { marginRight: 'auto', marginLeft: 'auto' } : {},
            generatePaddingFor(0)
          ),
        },
        ...atRules,
      ],
      variants('container')
    )
  }
}

function accessibility () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.sr-only': {
          position: 'absolute',
          width: '1px',
          height: '1px',
          padding: '0',
          margin: '-1px',
          overflow: 'hidden',
          clip: 'rect(0, 0, 0, 0)',
          whiteSpace: 'nowrap',
          borderWidth: '0',
        },
        '.not-sr-only': {
          position: 'static',
          width: 'auto',
          height: 'auto',
          padding: '0',
          margin: '0',
          overflow: 'visible',
          clip: 'auto',
          whiteSpace: 'normal',
        },
      },
      variants('accessibility')
    )
  }
}

function pointerEvents () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.pointer-events-none': { 'pointer-events': 'none' },
        '.pointer-events-auto': { 'pointer-events': 'auto' },
      },
      variants('pointerEvents')
    )
  }
}

function visibility () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.visible': { visibility: 'visible' },
        '.invisible': { visibility: 'hidden' },
      },
      variants('visibility')
    )
  }
}

function position () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.static': { position: 'static' },
        '.fixed': { position: 'fixed' },
        '.absolute': { position: 'absolute' },
        '.relative': { position: 'relative' },
        '.sticky': {
          position: 'sticky',
        },
      },
      variants('position')
    )
  }
}

function inset () {
  return function ({ matchUtilities, theme, variants }) {
    let options = {
      values: theme('inset'),
      variants: variants('inset'),
      type: 'any',
    }

    matchUtilities(
      {
        inset: (value) => {
          return { top: value, right: value, bottom: value, left: value }
        },
      },
      options
    )

    matchUtilities(
      {
        'inset-x': (value) => {
          return { left: value, right: value }
        },
        'inset-y': (value) => {
          return { top: value, bottom: value }
        },
      },
      options
    )

    matchUtilities(
      {
        top: (value) => {
          return { top: value }
        },
        right: (value) => {
          return { right: value }
        },
        bottom: (value) => {
          return { bottom: value }
        },
        left: (value) => {
          return { left: value }
        },
      },
      options
    )
  }
}

function isolation () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.isolate': {
          isolation: 'isolate',
        },
        '.isolation-auto': {
          isolation: 'auto',
        },
      },
      variants('isolation')
    )
  }
}



function zIndex () {
  return createUtilityPlugin('zIndex', [['z', ['zIndex']]])
}



function order () {
  return createUtilityPlugin('order')
}



function gridColumn () {
  return createUtilityPlugin('gridColumn', [['col', ['gridColumn']]])
}



function gridColumnStart () {
  return createUtilityPlugin('gridColumnStart', [['col-start', ['gridColumnStart']]])
}



function gridColumnEnd () {
  return createUtilityPlugin('gridColumnEnd', [['col-end', ['gridColumnEnd']]])
}



function gridRow () {
  return createUtilityPlugin('gridRow', [['row', ['gridRow']]])
}



function gridRowStart () {
  return createUtilityPlugin('gridRowStart', [['row-start', ['gridRowStart']]])
}



function gridRowEnd () {
  return createUtilityPlugin('gridRowEnd', [['row-end', ['gridRowEnd']]])
}

function float () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.float-right': { float: 'right' },
        '.float-left': { float: 'left' },
        '.float-none': { float: 'none' },
      },
      variants('float')
    )
  }
}

function clear () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.clear-left': { clear: 'left' },
        '.clear-right': { clear: 'right' },
        '.clear-both': { clear: 'both' },
        '.clear-none': { clear: 'none' },
      },
      variants('clear')
    )
  }
}



function margin () {
  return createUtilityPlugin('margin', [
    ['m', ['margin']],
    [
      ['mx', ['margin-left', 'margin-right']],
      ['my', ['margin-top', 'margin-bottom']],
    ],
    [
      ['mt', ['margin-top']],
      ['mr', ['margin-right']],
      ['mb', ['margin-bottom']],
      ['ml', ['margin-left']],
    ],
  ])
}

function boxSizing () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.box-border': { 'box-sizing': 'border-box' },
        '.box-content': { 'box-sizing': 'content-box' },
      },
      variants('boxSizing')
    )
  }
}

function display () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.block': {
          display: 'block',
        },
        '.inline-block': {
          display: 'inline-block',
        },
        '.inline': {
          display: 'inline',
        },
        '.flex': {
          display: 'flex',
        },
        '.inline-flex': {
          display: 'inline-flex',
        },
        '.table': {
          display: 'table',
        },
        '.inline-table': {
          display: 'inline-table',
        },
        '.table-caption': {
          display: 'table-caption',
        },
        '.table-cell': {
          display: 'table-cell',
        },
        '.table-column': {
          display: 'table-column',
        },
        '.table-column-group': {
          display: 'table-column-group',
        },
        '.table-footer-group': {
          display: 'table-footer-group',
        },
        '.table-header-group': {
          display: 'table-header-group',
        },
        '.table-row-group': {
          display: 'table-row-group',
        },
        '.table-row': {
          display: 'table-row',
        },
        '.flow-root': {
          display: 'flow-root',
        },
        '.grid': {
          display: 'grid',
        },
        '.inline-grid': {
          display: 'inline-grid',
        },
        '.contents': {
          display: 'contents',
        },
        '.list-item': {
          display: 'list-item',
        },
        '.hidden': {
          display: 'none',
        },
      },
      variants('display')
    )
  }
}



function height () {
  return createUtilityPlugin('height', [['h', ['height']]])
}



function maxHeight () {
  return createUtilityPlugin('maxHeight', [['max-h', ['maxHeight']]])
}



function minHeight () {
  return createUtilityPlugin('minHeight', [['min-h', ['minHeight']]])
}



function width () {
  return createUtilityPlugin('width', [['w', ['width']]])
}



function minWidth () {
  return createUtilityPlugin('minWidth', [['min-w', ['minWidth']]])
}



function maxWidth () {
  return createUtilityPlugin('maxWidth', [['max-w', ['maxWidth']]])
}



function flex () {
  return createUtilityPlugin('flex')
}



function flexShrink () {
  return createUtilityPlugin('flexShrink', [['flex-shrink', ['flex-shrink']]])
}



function flexGrow () {
  return createUtilityPlugin('flexGrow', [['flex-grow', ['flex-grow']]])
}

function tableLayout () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.table-auto': { 'table-layout': 'auto' },
        '.table-fixed': { 'table-layout': 'fixed' },
      },
      variants('tableLayout')
    )
  }
}

function borderCollapse () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.border-collapse': { 'border-collapse': 'collapse' },
        '.border-separate': { 'border-collapse': 'separate' },
      },
      variants('borderCollapse')
    )
  }
}



function transformOrigin () {
  return createUtilityPlugin('transformOrigin', [['origin', ['transformOrigin']]])
}

function transform () {
  return function ({ config, addBase, addUtilities, variants }) {
    if (config('mode') === 'jit') {
      addBase({
        '@defaults transform': {
          '--tw-translate-x': '0',
          '--tw-translate-y': '0',
          '--tw-rotate': '0',
          '--tw-skew-x': '0',
          '--tw-skew-y': '0',
          '--tw-scale-x': '1',
          '--tw-scale-y': '1',
          '--tw-transform': [
            'translateX(var(--tw-translate-x))',
            'translateY(var(--tw-translate-y))',
            'rotate(var(--tw-rotate))',
            'skewX(var(--tw-skew-x))',
            'skewY(var(--tw-skew-y))',
            'scaleX(var(--tw-scale-x))',
            'scaleY(var(--tw-scale-y))',
          ].join(' '),
        },
      })
      addUtilities(
        {
          '.transform': {
            '@defaults transform': {},
            transform: 'var(--tw-transform)',
          },
          '.transform-cpu': {
            '--tw-transform': [
              'translateX(var(--tw-translate-x))',
              'translateY(var(--tw-translate-y))',
              'rotate(var(--tw-rotate))',
              'skewX(var(--tw-skew-x))',
              'skewY(var(--tw-skew-y))',
              'scaleX(var(--tw-scale-x))',
              'scaleY(var(--tw-scale-y))',
            ].join(' '),
          },
          '.transform-gpu': {
            '--tw-transform': [
              'translate3d(var(--tw-translate-x), var(--tw-translate-y), 0)',
              'rotate(var(--tw-rotate))',
              'skewX(var(--tw-skew-x))',
              'skewY(var(--tw-skew-y))',
              'scaleX(var(--tw-scale-x))',
              'scaleY(var(--tw-scale-y))',
            ].join(' '),
          },
          '.transform-none': { transform: 'none' },
        },
        variants('transform')
      )
    } else {
      addUtilities(
        {
          '.transform': {
            '--tw-translate-x': '0',
            '--tw-translate-y': '0',
            '--tw-rotate': '0',
            '--tw-skew-x': '0',
            '--tw-skew-y': '0',
            '--tw-scale-x': '1',
            '--tw-scale-y': '1',
            transform: [
              'translateX(var(--tw-translate-x))',
              'translateY(var(--tw-translate-y))',
              'rotate(var(--tw-rotate))',
              'skewX(var(--tw-skew-x))',
              'skewY(var(--tw-skew-y))',
              'scaleX(var(--tw-scale-x))',
              'scaleY(var(--tw-scale-y))',
            ].join(' '),
          },
          '.transform-gpu': {
            '--tw-translate-x': '0',
            '--tw-translate-y': '0',
            '--tw-rotate': '0',
            '--tw-skew-x': '0',
            '--tw-skew-y': '0',
            '--tw-scale-x': '1',
            '--tw-scale-y': '1',
            transform: [
              'translate3d(var(--tw-translate-x), var(--tw-translate-y), 0)',
              'rotate(var(--tw-rotate))',
              'skewX(var(--tw-skew-x))',
              'skewY(var(--tw-skew-y))',
              'scaleX(var(--tw-scale-x))',
              'scaleY(var(--tw-scale-y))',
            ].join(' '),
          },
          '.transform-none': { transform: 'none' },
        },
        variants('transform')
      )
    }
  }
}



function translate () {
  return function ({ config, ...rest }) {
    if (config('mode') === 'jit') {
      return createUtilityPlugin('translate', [
        [
          [
            'translate-x',
            [['@defaults transform', {}], '--tw-translate-x', ['transform', 'var(--tw-transform)']],
          ],
          [
            'translate-y',
            [['@defaults transform', {}], '--tw-translate-y', ['transform', 'var(--tw-transform)']],
          ],
        ],
      ])({ config, ...rest })
    } else {
      return createUtilityPlugin('translate', [
        [
          ['translate-x', ['--tw-translate-x']],
          ['translate-y', ['--tw-translate-y']],
        ],
      ])({ config, ...rest })
    }
  }
}



function rotate () {
  return function ({ config, ...rest }) {
    if (config('mode') === 'jit') {
      return createUtilityPlugin('rotate', [
        [
          'rotate',
          [['@defaults transform', {}], '--tw-rotate', ['transform', 'var(--tw-transform)']],
        ],
      ])({ config, ...rest })
    } else {
      return createUtilityPlugin('rotate', [['rotate', ['--tw-rotate']]])({ config, ...rest })
    }
  }
}



function skew () {
  return function ({ config, ...rest }) {
    if (config('mode') === 'jit') {
      return createUtilityPlugin('skew', [
        [
          [
            'skew-x',
            [['@defaults transform', {}], '--tw-skew-x', ['transform', 'var(--tw-transform)']],
          ],
          [
            'skew-y',
            [['@defaults transform', {}], '--tw-skew-y', ['transform', 'var(--tw-transform)']],
          ],
        ],
      ])({ config, ...rest })
    } else {
      return createUtilityPlugin('skew', [
        [
          ['skew-x', ['--tw-skew-x']],
          ['skew-y', ['--tw-skew-y']],
        ],
      ])({ config, ...rest })
    }
  }
}



function scale () {
  return function ({ config, ...rest }) {
    if (config('mode') === 'jit') {
      return createUtilityPlugin('scale', [
        [
          'scale',
          [
            ['@defaults transform', {}],
            '--tw-scale-x',
            '--tw-scale-y',
            ['transform', 'var(--tw-transform)'],
          ],
        ],
        [
          [
            'scale-x',
            [['@defaults transform', {}], '--tw-scale-x', ['transform', 'var(--tw-transform)']],
          ],
          [
            'scale-y',
            [['@defaults transform', {}], '--tw-scale-y', ['transform', 'var(--tw-transform)']],
          ],
        ],
      ])({ config, ...rest })
    } else {
      return createUtilityPlugin('scale', [
        ['scale', ['--tw-scale-x', '--tw-scale-y']],
        [
          ['scale-x', ['--tw-scale-x']],
          ['scale-y', ['--tw-scale-y']],
        ],
      ])({ config, ...rest })
    }
  }
}



function animation () {
  return function ({ matchUtilities, theme, variants, prefix }) {
    let prefixName = (name) => prefix(`.${name}`).slice(1)
    let keyframes = Object.fromEntries(
      Object.entries(theme('keyframes')).map(([key, value]) => {
        return [
          key,
          [
            {
              [`@keyframes ${prefixName(key)}`]: value,
            },
            { respectVariants: false },
          ],
        ]
      })
    )

    matchUtilities(
      {
        animate: (value, { includeRules }) => {
          let { name: animationName } = parseAnimationValue(value)

          if (keyframes[animationName] !== undefined) {
            includeRules(keyframes[animationName], { respectImportant: false })
          }

          if (animationName === undefined || keyframes[animationName] === undefined) {
            return { animation: value }
          }

          return {
            animation: value.replace(animationName, prefixName(animationName)),
          }
        },
      },
      { values: theme('animation'), variants: variants('animation') }
    )
  }
}



function cursor () {
  return createUtilityPlugin('cursor')
}

function userSelect () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.select-none': { 'user-select': 'none' },
        '.select-text': { 'user-select': 'text' },
        '.select-all': { 'user-select': 'all' },
        '.select-auto': { 'user-select': 'auto' },
      },
      variants('userSelect')
    )
  }
}

function resize () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.resize-none': { resize: 'none' },
        '.resize-y': { resize: 'vertical' },
        '.resize-x': { resize: 'horizontal' },
        '.resize': { resize: 'both' },
      },
      variants('resize')
    )
  }
}

function listStylePosition () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.list-inside': { 'list-style-position': 'inside' },
        '.list-outside': { 'list-style-position': 'outside' },
      },
      variants('listStylePosition')
    )
  }
}



function listStyleType () {
  return createUtilityPlugin('listStyleType', [['list', ['listStyleType']]])
}

function appearance () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.appearance-none': { appearance: 'none' },
      },
      variants('appearance')
    )
  }
}



function gridAutoColumns () {
  return createUtilityPlugin('gridAutoColumns', [['auto-cols', ['gridAutoColumns']]])
}

function gridAutoFlow () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.grid-flow-row': { gridAutoFlow: 'row' },
        '.grid-flow-col': { gridAutoFlow: 'column' },
        '.grid-flow-row-dense': { gridAutoFlow: 'row dense' },
        '.grid-flow-col-dense': { gridAutoFlow: 'column dense' },
      },
      variants('gridAutoFlow')
    )
  }
}



function gridAutoRows () {
  return createUtilityPlugin('gridAutoRows', [['auto-rows', ['gridAutoRows']]])
}




function gridTemplateColumns () {
  return createUtilityPlugin('gridTemplateColumns', [['grid-cols', ['gridTemplateColumns']]], {
    resolveArbitraryValue: asList,
  })
}




function gridTemplateRows () {
  return createUtilityPlugin('gridTemplateRows', [['grid-rows', ['gridTemplateRows']]], {
    resolveArbitraryValue: asList,
  })
}

function flexDirection () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.flex-row': {
          'flex-direction': 'row',
        },
        '.flex-row-reverse': {
          'flex-direction': 'row-reverse',
        },
        '.flex-col': {
          'flex-direction': 'column',
        },
        '.flex-col-reverse': {
          'flex-direction': 'column-reverse',
        },
      },
      variants('flexDirection')
    )
  }
}

function flexWrap () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.flex-wrap': {
          'flex-wrap': 'wrap',
        },
        '.flex-wrap-reverse': {
          'flex-wrap': 'wrap-reverse',
        },
        '.flex-nowrap': {
          'flex-wrap': 'nowrap',
        },
      },
      variants('flexWrap')
    )
  }
}

function placeContent () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.place-content-center': {
          'place-content': 'center',
        },
        '.place-content-start': {
          'place-content': 'start',
        },
        '.place-content-end': {
          'place-content': 'end',
        },
        '.place-content-between': {
          'place-content': 'space-between',
        },
        '.place-content-around': {
          'place-content': 'space-around',
        },
        '.place-content-evenly': {
          'place-content': 'space-evenly',
        },
        '.place-content-stretch': {
          'place-content': 'stretch',
        },
      },
      variants('placeContent')
    )
  }
}

function placeItems () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.place-items-start': {
          'place-items': 'start',
        },
        '.place-items-end': {
          'place-items': 'end',
        },
        '.place-items-center': {
          'place-items': 'center',
        },
        '.place-items-stretch': {
          'place-items': 'stretch',
        },
      },
      variants('placeItems')
    )
  }
}

function alignContent () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.content-center': {
          'align-content': 'center',
        },
        '.content-start': {
          'align-content': 'flex-start',
        },
        '.content-end': {
          'align-content': 'flex-end',
        },
        '.content-between': {
          'align-content': 'space-between',
        },
        '.content-around': {
          'align-content': 'space-around',
        },
        '.content-evenly': {
          'align-content': 'space-evenly',
        },
      },
      variants('alignContent')
    )
  }
}

function alignItems () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.items-start': {
          'align-items': 'flex-start',
        },
        '.items-end': {
          'align-items': 'flex-end',
        },
        '.items-center': {
          'align-items': 'center',
        },
        '.items-baseline': {
          'align-items': 'baseline',
        },
        '.items-stretch': {
          'align-items': 'stretch',
        },
      },
      variants('alignItems')
    )
  }
}

function justifyContent () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.justify-start': {
          'justify-content': 'flex-start',
        },
        '.justify-end': {
          'justify-content': 'flex-end',
        },
        '.justify-center': {
          'justify-content': 'center',
        },
        '.justify-between': {
          'justify-content': 'space-between',
        },
        '.justify-around': {
          'justify-content': 'space-around',
        },
        '.justify-evenly': {
          'justify-content': 'space-evenly',
        },
      },
      variants('justifyContent')
    )
  }
}

function justifyItems () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.justify-items-start': {
          'justify-items': 'start',
        },
        '.justify-items-end': {
          'justify-items': 'end',
        },
        '.justify-items-center': {
          'justify-items': 'center',
        },
        '.justify-items-stretch': {
          'justify-items': 'stretch',
        },
      },
      variants('justifyItems')
    )
  }
}



function gap () {
  return createUtilityPlugin('gap', [
    ['gap', ['gap']],
    [
      ['gap-x', ['columnGap']],
      ['gap-y', ['rowGap']],
    ],
  ])
}

function space () {
  return function ({ matchUtilities, addUtilities, theme, variants }) {
    matchUtilities(
      {
        'space-x': (value) => {
          value = value === '0' ? '0px' : value

          return {
            '& > :not([hidden]) ~ :not([hidden])': {
              '--tw-space-x-reverse': '0',
              'margin-right': `calc(${value} * var(--tw-space-x-reverse))`,
              'margin-left': `calc(${value} * calc(1 - var(--tw-space-x-reverse)))`,
            },
          }
        },
        'space-y': (value) => {
          value = value === '0' ? '0px' : value

          return {
            '& > :not([hidden]) ~ :not([hidden])': {
              '--tw-space-y-reverse': '0',
              'margin-top': `calc(${value} * calc(1 - var(--tw-space-y-reverse)))`,
              'margin-bottom': `calc(${value} * var(--tw-space-y-reverse))`,
            },
          }
        },
      },
      {
        values: theme('space'),
        variants: variants('space'),
        type: 'any',
      }
    )

    addUtilities(
      {
        '.space-y-reverse > :not([hidden]) ~ :not([hidden])': {
          '--tw-space-y-reverse': '1',
        },
        '.space-x-reverse > :not([hidden]) ~ :not([hidden])': {
          '--tw-space-x-reverse': '1',
        },
      },
      variants('space')
    )
  }
}

function divideWidth () {
  return function ({ matchUtilities, addUtilities, theme, variants }) {
    matchUtilities(
      {
        'divide-x': (value) => {
          value = value === '0' ? '0px' : value

          return {
            '& > :not([hidden]) ~ :not([hidden])': {
              '--tw-divide-x-reverse': '0',
              'border-right-width': `calc(${value} * var(--tw-divide-x-reverse))`,
              'border-left-width': `calc(${value} * calc(1 - var(--tw-divide-x-reverse)))`,
            },
          }
        },
        'divide-y': (value) => {
          value = value === '0' ? '0px' : value

          return {
            '& > :not([hidden]) ~ :not([hidden])': {
              '--tw-divide-y-reverse': '0',
              'border-top-width': `calc(${value} * calc(1 - var(--tw-divide-y-reverse)))`,
              'border-bottom-width': `calc(${value} * var(--tw-divide-y-reverse))`,
            },
          }
        },
      },
      {
        values: theme('divideWidth'),
        variants: variants('divideWidth'),
        type: 'length',
      }
    )

    addUtilities(
      {
        '.divide-y-reverse > :not([hidden]) ~ :not([hidden])': {
          '--tw-divide-y-reverse': '1',
        },
        '.divide-x-reverse > :not([hidden]) ~ :not([hidden])': {
          '--tw-divide-x-reverse': '1',
        },
      },
      variants('divideWidth')
    )
  }
}

function divideStyle () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.divide-solid > :not([hidden]) ~ :not([hidden])': {
          'border-style': 'solid',
        },
        '.divide-dashed > :not([hidden]) ~ :not([hidden])': {
          'border-style': 'dashed',
        },
        '.divide-dotted > :not([hidden]) ~ :not([hidden])': {
          'border-style': 'dotted',
        },
        '.divide-double > :not([hidden]) ~ :not([hidden])': {
          'border-style': 'double',
        },
        '.divide-none > :not([hidden]) ~ :not([hidden])': {
          'border-style': 'none',
        },
      },
      variants('divideStyle')
    )
  }
}




function divideColor () {
  return function ({ matchUtilities, theme, variants, corePlugins }) {
    matchUtilities(
      {
        divide: (value) => {
          if (!corePlugins('divideOpacity')) {
            return {
              ['& > :not([hidden]) ~ :not([hidden])']: {
                'border-color': value,
              },
            }
          }

          return {
            ['& > :not([hidden]) ~ :not([hidden])']: withAlphaVariable({
              color: value,
              property: 'border-color',
              variable: '--tw-divide-opacity',
            }),
          }
        },
      },
      {
        values: (({ DEFAULT: _, ...colors }) => colors)(flattenColorPalette(theme('divideColor'))),
        variants: variants('divideColor'),
        type: 'color',
      }
    )
  }
}

function divideOpacity () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'divide-opacity': (value) => {
          return {
            [`& > :not([hidden]) ~ :not([hidden])`]: {
              '--tw-divide-opacity': value,
            },
          }
        },
      },
      {
        values: theme('divideOpacity'),
        variants: variants('divideOpacity'),
        type: 'any',
      }
    )
  }
}

function placeSelf () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.place-self-auto': {
          'place-self': 'auto',
        },
        '.place-self-start': {
          'place-self': 'start',
        },
        '.place-self-end': {
          'place-self': 'end',
        },
        '.place-self-center': {
          'place-self': 'center',
        },
        '.place-self-stretch': {
          'place-self': 'stretch',
        },
      },
      variants('placeSelf')
    )
  }
}

function alignSelf () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.self-auto': {
          'align-self': 'auto',
        },
        '.self-start': {
          'align-self': 'flex-start',
        },
        '.self-end': {
          'align-self': 'flex-end',
        },
        '.self-center': {
          'align-self': 'center',
        },
        '.self-stretch': {
          'align-self': 'stretch',
        },
        '.self-baseline': {
          'align-self': 'baseline',
        },
      },
      variants('alignSelf')
    )
  }
}

function justifySelf () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.justify-self-auto': {
          'justify-self': 'auto',
        },
        '.justify-self-start': {
          'justify-self': 'start',
        },
        '.justify-self-end': {
          'justify-self': 'end',
        },
        '.justify-self-center': {
          'justify-self': 'center',
        },
        '.justify-self-stretch': {
          'justify-self': 'stretch',
        },
      },
      variants('justifySelf')
    )
  }
}

function overflow () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.overflow-auto': { overflow: 'auto' },
        '.overflow-hidden': { overflow: 'hidden' },
        '.overflow-visible': { overflow: 'visible' },
        '.overflow-scroll': { overflow: 'scroll' },
        '.overflow-x-auto': { 'overflow-x': 'auto' },
        '.overflow-y-auto': { 'overflow-y': 'auto' },
        '.overflow-x-hidden': { 'overflow-x': 'hidden' },
        '.overflow-y-hidden': { 'overflow-y': 'hidden' },
        '.overflow-x-visible': { 'overflow-x': 'visible' },
        '.overflow-y-visible': { 'overflow-y': 'visible' },
        '.overflow-x-scroll': { 'overflow-x': 'scroll' },
        '.overflow-y-scroll': { 'overflow-y': 'scroll' },
      },
      variants('overflow')
    )
  }
}

function overscrollBehavior () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.overscroll-auto': { 'overscroll-behavior': 'auto' },
        '.overscroll-contain': { 'overscroll-behavior': 'contain' },
        '.overscroll-none': { 'overscroll-behavior': 'none' },
        '.overscroll-y-auto': { 'overscroll-behavior-y': 'auto' },
        '.overscroll-y-contain': { 'overscroll-behavior-y': 'contain' },
        '.overscroll-y-none': { 'overscroll-behavior-y': 'none' },
        '.overscroll-x-auto': { 'overscroll-behavior-x': 'auto' },
        '.overscroll-x-contain': { 'overscroll-behavior-x': 'contain' },
        '.overscroll-x-none': { 'overscroll-behavior-x': 'none' },
      },
      variants('overscrollBehavior')
    )
  }
}

function textOverflow () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.truncate': {
          overflow: 'hidden',
          'text-overflow': 'ellipsis',
          'white-space': 'nowrap',
        },
        '.overflow-ellipsis': { 'text-overflow': 'ellipsis' },
        '.overflow-clip': { 'text-overflow': 'clip' },
      },
      variants('textOverflow')
    )
  }
}

function whitespace () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.whitespace-normal': { 'white-space': 'normal' },
        '.whitespace-nowrap': { 'white-space': 'nowrap' },
        '.whitespace-pre': { 'white-space': 'pre' },
        '.whitespace-pre-line': { 'white-space': 'pre-line' },
        '.whitespace-pre-wrap': { 'white-space': 'pre-wrap' },
      },
      variants('whitespace')
    )
  }
}

function wordBreak () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.break-normal': {
          'overflow-wrap': 'normal',
          'word-break': 'normal',
        },
        '.break-words': {
          'overflow-wrap': 'break-word',
        },
        '.break-all': { 'word-break': 'break-all' },
      },
      variants('wordBreak')
    )
  }
}



function borderRadius () {
  return createUtilityPlugin('borderRadius', [
    ['rounded', ['border-radius']],
    [
      ['rounded-t', ['border-top-left-radius', 'border-top-right-radius']],
      ['rounded-r', ['border-top-right-radius', 'border-bottom-right-radius']],
      ['rounded-b', ['border-bottom-right-radius', 'border-bottom-left-radius']],
      ['rounded-l', ['border-top-left-radius', 'border-bottom-left-radius']],
    ],
    [
      ['rounded-tl', ['border-top-left-radius']],
      ['rounded-tr', ['border-top-right-radius']],
      ['rounded-br', ['border-bottom-right-radius']],
      ['rounded-bl', ['border-bottom-left-radius']],
    ],
  ])
}




function borderWidth () {
  return function (helpers) {
    if (helpers.config('mode') === 'jit') {
      createUtilityPlugin(
        'borderWidth',
        [
          ['border', [['@defaults border-width', {}], 'border-width']],
          [
            ['border-t', [['@defaults border-width', {}], 'border-top-width']],
            ['border-r', [['@defaults border-width', {}], 'border-right-width']],
            ['border-b', [['@defaults border-width', {}], 'border-bottom-width']],
            ['border-l', [['@defaults border-width', {}], 'border-left-width']],
          ],
        ],
        { resolveArbitraryValue: asLength }
      )(helpers)
    } else {
      createUtilityPlugin(
        'borderWidth',
        [
          ['border', ['border-width']],
          [
            ['border-t', ['border-top-width']],
            ['border-r', ['border-right-width']],
            ['border-b', ['border-bottom-width']],
            ['border-l', ['border-left-width']],
          ],
        ],
        { resolveArbitraryValue: asLength }
      )(helpers)
    }
  }
}

function borderStyle () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.border-solid': {
          'border-style': 'solid',
        },
        '.border-dashed': {
          'border-style': 'dashed',
        },
        '.border-dotted': {
          'border-style': 'dotted',
        },
        '.border-double': {
          'border-style': 'double',
        },
        '.border-none': {
          'border-style': 'none',
        },
      },
      variants('borderStyle')
    )
  }
}




function borderColor () {
  return function ({ config, addBase, matchUtilities, theme, variants, corePlugins }) {
    if (config('mode') === 'jit') {
      if (!corePlugins('borderOpacity')) {
        addBase({
          '@defaults border-width': {
            'border-color': theme('borderColor.DEFAULT', 'currentColor'),
          },
        })
      } else {
        addBase({
          '@defaults border-width': withAlphaVariable({
            color: theme('borderColor.DEFAULT', 'currentColor'),
            property: 'border-color',
            variable: '--tw-border-opacity',
          }),
        })
      }
    } else {
      if (!corePlugins('borderOpacity')) {
        addBase({
          '*, ::before, ::after': {
            'border-color': theme('borderColor.DEFAULT', 'currentColor'),
          },
        })
      } else {
        addBase({
          '*, ::before, ::after': withAlphaVariable({
            color: theme('borderColor.DEFAULT', 'currentColor'),
            property: 'border-color',
            variable: '--tw-border-opacity',
          }),
        })
      }
    }

    matchUtilities(
      {
        border: (value) => {
          if (!corePlugins('borderOpacity')) {
            return {
              'border-color': value,
            }
          }

          return withAlphaVariable({
            color: value,
            property: 'border-color',
            variable: '--tw-border-opacity',
          })
        },
      },
      {
        values: (({ DEFAULT: _, ...colors }) => colors)(flattenColorPalette(theme('borderColor'))),
        variants: variants('borderColor'),
        type: 'color',
      }
    )

    if (config('mode') === 'jit') {
      matchUtilities(
        {
          'border-t': (value) => {
            if (!corePlugins('borderOpacity')) {
              return {
                'border-top-color': value,
              }
            }

            return withAlphaVariable({
              color: value,
              property: 'border-top-color',
              variable: '--tw-border-opacity',
            })
          },
          'border-r': (value) => {
            if (!corePlugins('borderOpacity')) {
              return {
                'border-right-color': value,
              }
            }

            return withAlphaVariable({
              color: value,
              property: 'border-right-color',
              variable: '--tw-border-opacity',
            })
          },
          'border-b': (value) => {
            if (!corePlugins('borderOpacity')) {
              return {
                'border-bottom-color': value,
              }
            }

            return withAlphaVariable({
              color: value,
              property: 'border-bottom-color',
              variable: '--tw-border-opacity',
            })
          },
          'border-l': (value) => {
            if (!corePlugins('borderOpacity')) {
              return {
                'border-left-color': value,
              }
            }

            return withAlphaVariable({
              color: value,
              property: 'border-left-color',
              variable: '--tw-border-opacity',
            })
          },
        },
        {
          values: (({ DEFAULT: _, ...colors }) => colors)(
            flattenColorPalette(theme('borderColor'))
          ),
          variants: variants('borderColor'),
          type: 'color',
        }
      )
    }
  }
}



function borderOpacity () {
  return createUtilityPlugin('borderOpacity', [['border-opacity', ['--tw-border-opacity']]])
}




function backgroundColor () {
  return function ({ matchUtilities, theme, variants, corePlugins }) {
    matchUtilities(
      {
        bg: (value) => {
          if (!corePlugins('backgroundOpacity')) {
            return {
              'background-color': value,
            }
          }

          return withAlphaVariable({
            color: value,
            property: 'background-color',
            variable: '--tw-bg-opacity',
          })
        },
      },
      {
        values: flattenColorPalette(theme('backgroundColor')),
        variants: variants('backgroundColor'),
        type: 'color',
      }
    )
  }
}



function backgroundOpacity () {
  return createUtilityPlugin('backgroundOpacity', [['bg-opacity', ['--tw-bg-opacity']]])
}




function backgroundImage () {
  return createUtilityPlugin('backgroundImage', [['bg', ['background-image']]], {
    resolveArbitraryValue: asLookupValue,
  })
}





function transparentTo(value) {
  return withAlphaValue(value, 0, 'rgba(255, 255, 255, 0)')
}

function gradientColorStops () {
  return function ({ matchUtilities, theme, variants }) {
    let options = {
      values: flattenColorPalette(theme('gradientColorStops')),
      variants: variants('gradientColorStops'),
      type: ['color', 'any'],
    }

    matchUtilities(
      {
        from: (value) => {
          let transparentToValue = transparentTo(value)

          return {
            '--tw-gradient-from': toColorValue(value, 'from'),
            '--tw-gradient-stops': `var(--tw-gradient-from), var(--tw-gradient-to, ${transparentToValue})`,
          }
        },
      },
      options
    )
    matchUtilities(
      {
        via: (value) => {
          let transparentToValue = transparentTo(value)

          return {
            '--tw-gradient-stops': `var(--tw-gradient-from), ${toColorValue(
              value,
              'via'
            )}, var(--tw-gradient-to, ${transparentToValue})`,
          }
        },
      },
      options
    )
    matchUtilities(
      {
        to: (value) => {
          return { '--tw-gradient-to': toColorValue(value, 'to') }
        },
      },
      options
    )
  }
}

function boxDecorationBreak () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.decoration-slice': {
          'box-decoration-break': 'slice',
        },
        '.decoration-clone': {
          'box-decoration-break': 'clone',
        },
      },
      variants('boxDecorationBreak')
    )
  }
}




function backgroundSize () {
  return createUtilityPlugin('backgroundSize', [['bg', ['background-size']]], {
    resolveArbitraryValue: asLookupValue,
  })
}

function backgroundAttachment () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.bg-fixed': { 'background-attachment': 'fixed' },
        '.bg-local': { 'background-attachment': 'local' },
        '.bg-scroll': { 'background-attachment': 'scroll' },
      },
      variants('backgroundAttachment')
    )
  }
}

function backgroundClip () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.bg-clip-border': { 'background-clip': 'border-box' },
        '.bg-clip-padding': { 'background-clip': 'padding-box' },
        '.bg-clip-content': { 'background-clip': 'content-box' },
        '.bg-clip-text': { 'background-clip': 'text' },
      },
      variants('backgroundClip')
    )
  }
}




function backgroundPosition () {
  return createUtilityPlugin('backgroundPosition', [['bg', ['background-position']]], {
    resolveArbitraryValue: asLookupValue,
  })
}

function backgroundRepeat () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.bg-repeat': { 'background-repeat': 'repeat' },
        '.bg-no-repeat': { 'background-repeat': 'no-repeat' },
        '.bg-repeat-x': { 'background-repeat': 'repeat-x' },
        '.bg-repeat-y': { 'background-repeat': 'repeat-y' },
        '.bg-repeat-round': { 'background-repeat': 'round' },
        '.bg-repeat-space': { 'background-repeat': 'space' },
      },
      variants('backgroundRepeat')
    )
  }
}

function backgroundOrigin () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.bg-origin-border': { 'background-origin': 'border-box' },
        '.bg-origin-padding': { 'background-origin': 'padding-box' },
        '.bg-origin-content': { 'background-origin': 'content-box' },
      },
      variants('backgroundOrigin')
    )
  }
}




function fill () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        fill: (value) => {
          return { fill: toColorValue(value) }
        },
      },
      {
        values: flattenColorPalette(theme('fill')),
        variants: variants('fill'),
        type: ['color', 'any'],
      }
    )
  }
}




function stroke () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        stroke: (value) => {
          return {
            stroke: toColorValue(value),
          }
        },
      },
      {
        values: flattenColorPalette(theme('stroke')),
        variants: variants('stroke'),
        type: 'color',
      }
    )
  }
}




function strokeWidth () {
  return createUtilityPlugin('strokeWidth', [['stroke', ['stroke-width']]], {
    resolveArbitraryValue: asLength,
  })
}

function objectFit () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.object-contain': { 'object-fit': 'contain' },
        '.object-cover': { 'object-fit': 'cover' },
        '.object-fill': { 'object-fit': 'fill' },
        '.object-none': { 'object-fit': 'none' },
        '.object-scale-down': { 'object-fit': 'scale-down' },
      },
      variants('objectFit')
    )
  }
}



function objectPosition () {
  return createUtilityPlugin('objectPosition', [['object', ['object-position']]])
}



function padding () {
  return createUtilityPlugin('padding', [
    ['p', ['padding']],
    [
      ['px', ['padding-left', 'padding-right']],
      ['py', ['padding-top', 'padding-bottom']],
    ],
    [
      ['pt', ['padding-top']],
      ['pr', ['padding-right']],
      ['pb', ['padding-bottom']],
      ['pl', ['padding-left']],
    ],
  ])
}

function textAlign () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.text-left': { 'text-align': 'left' },
        '.text-center': { 'text-align': 'center' },
        '.text-right': { 'text-align': 'right' },
        '.text-justify': { 'text-align': 'justify' },
      },
      variants('textAlign')
    )
  }
}

function verticalAlign () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.align-baseline': { 'vertical-align': 'baseline' },
        '.align-top': { 'vertical-align': 'top' },
        '.align-middle': { 'vertical-align': 'middle' },
        '.align-bottom': { 'vertical-align': 'bottom' },
        '.align-text-top': { 'vertical-align': 'text-top' },
        '.align-text-bottom': { 'vertical-align': 'text-bottom' },
      },
      variants('verticalAlign')
    )
  }
}




function fontFamily () {
  return createUtilityPlugin('fontFamily', [['font', ['fontFamily']]], {
    resolveArbitraryValue: asLookupValue,
  })
}



function fontSize () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        text: (value) => {
          let [fontSize, options] = Array.isArray(value) ? value : [value]
          let { lineHeight, letterSpacing } = isPlainObject(options)
            ? options
            : { lineHeight: options }

          return {
            'font-size': fontSize,
            ...(lineHeight === undefined ? {} : { 'line-height': lineHeight }),
            ...(letterSpacing === undefined ? {} : { 'letter-spacing': letterSpacing }),
          }
        },
      },
      {
        values: theme('fontSize'),
        variants: variants('fontSize'),
        type: 'length',
      }
    )
  }
}




function fontWeight () {
  return createUtilityPlugin('fontWeight', [['font', ['fontWeight']]], {
    resolveArbitraryValue: asLookupValue,
  })
}

function textTransform () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.uppercase': { 'text-transform': 'uppercase' },
        '.lowercase': { 'text-transform': 'lowercase' },
        '.capitalize': { 'text-transform': 'capitalize' },
        '.normal-case': { 'text-transform': 'none' },
      },
      variants('textTransform')
    )
  }
}

function fontStyle () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.italic': { 'font-style': 'italic' },
        '.not-italic': { 'font-style': 'normal' },
      },
      variants('fontStyle')
    )
  }
}

function fontVariantNumeric () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.ordinal, .slashed-zero, .lining-nums, .oldstyle-nums, .proportional-nums, .tabular-nums, .diagonal-fractions, .stacked-fractions':
          {
            '--tw-ordinal': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-slashed-zero': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-numeric-figure': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-numeric-spacing': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-numeric-fraction': 'var(--tw-empty,/*!*/ /*!*/)',
            'font-variant-numeric':
              'var(--tw-ordinal) var(--tw-slashed-zero) var(--tw-numeric-figure) var(--tw-numeric-spacing) var(--tw-numeric-fraction)',
          },
        '.normal-nums': {
          'font-variant-numeric': 'normal',
        },
        '.ordinal': {
          '--tw-ordinal': 'ordinal',
        },
        '.slashed-zero': {
          '--tw-slashed-zero': 'slashed-zero',
        },
        '.lining-nums': {
          '--tw-numeric-figure': 'lining-nums',
        },
        '.oldstyle-nums': {
          '--tw-numeric-figure': 'oldstyle-nums',
        },
        '.proportional-nums': {
          '--tw-numeric-spacing': 'proportional-nums',
        },
        '.tabular-nums': {
          '--tw-numeric-spacing': 'tabular-nums',
        },
        '.diagonal-fractions': {
          '--tw-numeric-fraction': 'diagonal-fractions',
        },
        '.stacked-fractions': {
          '--tw-numeric-fraction': 'stacked-fractions',
        },
      },
      variants('fontVariantNumeric')
    )
  }
}



function lineHeight () {
  return createUtilityPlugin('lineHeight', [['leading', ['lineHeight']]])
}



function letterSpacing () {
  return createUtilityPlugin('letterSpacing', [['tracking', ['letterSpacing']]])
}




function textColor () {
  return function ({ matchUtilities, theme, variants, corePlugins }) {
    matchUtilities(
      {
        text: (value) => {
          if (!corePlugins('textOpacity')) {
            return { color: value }
          }

          return withAlphaVariable({
            color: value,
            property: 'color',
            variable: '--tw-text-opacity',
          })
        },
      },
      {
        values: flattenColorPalette(theme('textColor')),
        variants: variants('textColor'),
        type: 'color',
      }
    )
  }
}



function textOpacity () {
  return createUtilityPlugin('textOpacity', [['text-opacity', ['--tw-text-opacity']]])
}

function textDecoration () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.underline': { 'text-decoration': 'underline' },
        '.line-through': { 'text-decoration': 'line-through' },
        '.no-underline': { 'text-decoration': 'none' },
      },
      variants('textDecoration')
    )
  }
}

function fontSmoothing () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.antialiased': {
          '-webkit-font-smoothing': 'antialiased',
          '-moz-osx-font-smoothing': 'grayscale',
        },
        '.subpixel-antialiased': {
          '-webkit-font-smoothing': 'auto',
          '-moz-osx-font-smoothing': 'auto',
        },
      },
      variants('fontSmoothing')
    )
  }
}




function placeholderColor () {
  return function ({ matchUtilities, theme, variants, corePlugins }) {
    matchUtilities(
      {
        placeholder: (value) => {
          if (!corePlugins('placeholderOpacity')) {
            return {
              '&::placeholder': {
                color: value,
              },
            }
          }

          return {
            '&::placeholder': withAlphaVariable({
              color: value,
              property: 'color',
              variable: '--tw-placeholder-opacity',
            }),
          }
        },
      },
      {
        values: flattenColorPalette(theme('placeholderColor')),
        variants: variants('placeholderColor'),
        type: ['color', 'any'],
      }
    )
  }
}

function placeholderOpacity () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'placeholder-opacity': (value) => {
          return { ['&::placeholder']: { '--tw-placeholder-opacity': value } }
        },
      },
      {
        values: theme('placeholderOpacity'),
        variants: variants('placeholderOpacity'),
        type: 'any',
      }
    )
  }
}




function caretColor () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        caret: (value) => {
          return { 'caret-color': toColorValue(value) }
        },
      },
      {
        values: flattenColorPalette(theme('caretColor')),
        variants: variants('caretColor'),
        type: ['color', 'any'],
      }
    )
  }
}



function opacity () {
  return createUtilityPlugin('opacity', [['opacity', ['opacity']]])
}

function backgroundBlendMode () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.bg-blend-normal': { 'background-blend-mode': 'normal' },
        '.bg-blend-multiply': { 'background-blend-mode': 'multiply' },
        '.bg-blend-screen': { 'background-blend-mode': 'screen' },
        '.bg-blend-overlay': { 'background-blend-mode': 'overlay' },
        '.bg-blend-darken': { 'background-blend-mode': 'darken' },
        '.bg-blend-lighten': { 'background-blend-mode': 'lighten' },
        '.bg-blend-color-dodge': { 'background-blend-mode': 'color-dodge' },
        '.bg-blend-color-burn': { 'background-blend-mode': 'color-burn' },
        '.bg-blend-hard-light': { 'background-blend-mode': 'hard-light' },
        '.bg-blend-soft-light': { 'background-blend-mode': 'soft-light' },
        '.bg-blend-difference': { 'background-blend-mode': 'difference' },
        '.bg-blend-exclusion': { 'background-blend-mode': 'exclusion' },
        '.bg-blend-hue': { 'background-blend-mode': 'hue' },
        '.bg-blend-saturation': { 'background-blend-mode': 'saturation' },
        '.bg-blend-color': { 'background-blend-mode': 'color' },
        '.bg-blend-luminosity': { 'background-blend-mode': 'luminosity' },
      },
      variants('backgroundBlendMode')
    )
  }
}

function mixBlendMode () {
  return function ({ addUtilities, variants }) {
    addUtilities(
      {
        '.mix-blend-normal': { 'mix-blend-mode': 'normal' },
        '.mix-blend-multiply': { 'mix-blend-mode': 'multiply' },
        '.mix-blend-screen': { 'mix-blend-mode': 'screen' },
        '.mix-blend-overlay': { 'mix-blend-mode': 'overlay' },
        '.mix-blend-darken': { 'mix-blend-mode': 'darken' },
        '.mix-blend-lighten': { 'mix-blend-mode': 'lighten' },
        '.mix-blend-color-dodge': { 'mix-blend-mode': 'color-dodge' },
        '.mix-blend-color-burn': { 'mix-blend-mode': 'color-burn' },
        '.mix-blend-hard-light': { 'mix-blend-mode': 'hard-light' },
        '.mix-blend-soft-light': { 'mix-blend-mode': 'soft-light' },
        '.mix-blend-difference': { 'mix-blend-mode': 'difference' },
        '.mix-blend-exclusion': { 'mix-blend-mode': 'exclusion' },
        '.mix-blend-hue': { 'mix-blend-mode': 'hue' },
        '.mix-blend-saturation': { 'mix-blend-mode': 'saturation' },
        '.mix-blend-color': { 'mix-blend-mode': 'color' },
        '.mix-blend-luminosity': { 'mix-blend-mode': 'luminosity' },
      },
      variants('mixBlendMode')
    )
  }
}



let transformValue = transformThemeValue('boxShadow')
let defaultBoxShadow = [
  `var(--tw-ring-offset-shadow, 0 0 #0000)`,
  `var(--tw-ring-shadow, 0 0 #0000)`,
  `var(--tw-shadow)`,
].join(', ')

function boxShadow () {
  return function ({ config, matchUtilities, addBase, addUtilities, theme, variants }) {
    if (config('mode') === 'jit') {
      addBase({
        '@defaults box-shadow': {
          '--tw-ring-offset-shadow': '0 0 #0000',
          '--tw-ring-shadow': '0 0 #0000',
          '--tw-shadow': '0 0 #0000',
        },
      })
    } else {
      addUtilities(
        {
          '*, ::before, ::after': {
            '--tw-shadow': '0 0 #0000',
          },
        },
        { respectImportant: false }
      )
    }

    matchUtilities(
      {
        shadow: (value) => {
          value = transformValue(value)

          return {
            ...(config('mode') === 'jit' ? { '@defaults box-shadow': {} } : {}),
            '--tw-shadow': value === 'none' ? '0 0 #0000' : value,
            'box-shadow': defaultBoxShadow,
          }
        },
      },
      {
        values: theme('boxShadow'),
        variants: variants('boxShadow'),
        type: 'lookup',
      }
    )
  }
}

function outline () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        outline: (value) => {
          let [outline, outlineOffset = '0'] = Array.isArray(value) ? value : [value]

          return { outline, 'outline-offset': outlineOffset }
        },
      },
      { values: theme('outline'), variants: variants('outline'), type: 'any' }
    )
  }
}



function ringWidth () {
  return function ({ config, matchUtilities, addBase, addUtilities, theme, variants }) {
    let ringOpacityDefault = theme('ringOpacity.DEFAULT', '0.5')
    let ringColorDefault = withAlphaValue(
      theme('ringColor.DEFAULT'),
      ringOpacityDefault,
      `rgba(147, 197, 253, ${ringOpacityDefault})`
    )

    if (config('mode') === 'jit') {
      addBase({
        '@defaults ring-width': {
          '--tw-ring-inset': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-ring-offset-width': theme('ringOffsetWidth.DEFAULT', '0px'),
          '--tw-ring-offset-color': theme('ringOffsetColor.DEFAULT', '#fff'),
          '--tw-ring-color': ringColorDefault,
          '--tw-ring-offset-shadow': '0 0 #0000',
          '--tw-ring-shadow': '0 0 #0000',
          '--tw-shadow': '0 0 #0000',
        },
      })
    } else {
      addUtilities(
        {
          '*, ::before, ::after': {
            '--tw-ring-inset': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-ring-offset-width': theme('ringOffsetWidth.DEFAULT', '0px'),
            '--tw-ring-offset-color': theme('ringOffsetColor.DEFAULT', '#fff'),
            '--tw-ring-color': ringColorDefault,
            '--tw-ring-offset-shadow': '0 0 #0000',
            '--tw-ring-shadow': '0 0 #0000',
          },
        },
        { respectImportant: false }
      )
    }

    matchUtilities(
      {
        ring: (value) => {
          return {
            ...(config('mode') === 'jit' ? { '@defaults ring-width': {} } : {}),
            '--tw-ring-offset-shadow': `var(--tw-ring-inset) 0 0 0 var(--tw-ring-offset-width) var(--tw-ring-offset-color)`,
            '--tw-ring-shadow': `var(--tw-ring-inset) 0 0 0 calc(${value} + var(--tw-ring-offset-width)) var(--tw-ring-color)`,
            'box-shadow': [
              `var(--tw-ring-offset-shadow)`,
              `var(--tw-ring-shadow)`,
              `var(--tw-shadow, 0 0 #0000)`,
            ].join(', '),
          }
        },
      },
      {
        values: theme('ringWidth'),
        variants: variants('ringWidth'),
        type: 'length',
      }
    )

    addUtilities(
      {
        '.ring-inset': {
          ...(config('mode') === 'jit' ? { '@defaults ring-width': {} } : {}),
          '--tw-ring-inset': 'inset',
        },
      },
      variants('ringWidth')
    )
  }
}




function ringColor () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        ring: (value) => {
          return withAlphaVariable({
            color: value,
            property: '--tw-ring-color',
            variable: '--tw-ring-opacity',
          })
        },
      },
      {
        values: Object.fromEntries(
          Object.entries(flattenColorPalette(theme('ringColor'))).filter(
            ([modifier]) => modifier !== 'DEFAULT'
          )
        ),
        variants: variants('ringColor'),
        type: 'color',
      }
    )
  }
}



function ringOpacity () {
  return createUtilityPlugin('ringOpacity', [['ring-opacity', ['--tw-ring-opacity']]], {
    filterDefault: true,
  })
}




function ringOffsetWidth () {
  return createUtilityPlugin('ringOffsetWidth', [['ring-offset', ['--tw-ring-offset-width']]], {
    resolveArbitraryValue: asLength,
  })
}




function ringOffsetColor () {
  return function ({ matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'ring-offset': (value) => {
          return {
            '--tw-ring-offset-color': toColorValue(value),
          }
        },
      },
      {
        values: flattenColorPalette(theme('ringOffsetColor')),
        variants: variants('ringOffsetColor'),
        type: 'color',
      }
    )
  }
}

function filter () {
  return function ({ config, addBase, addUtilities, variants }) {
    if (config('mode') === 'jit') {
      addBase({
        '@defaults filter': {
          '--tw-blur': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-brightness': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-contrast': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-grayscale': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-hue-rotate': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-invert': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-saturate': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-sepia': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-drop-shadow': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-filter': [
            'var(--tw-blur)',
            'var(--tw-brightness)',
            'var(--tw-contrast)',
            'var(--tw-grayscale)',
            'var(--tw-hue-rotate)',
            'var(--tw-invert)',
            'var(--tw-saturate)',
            'var(--tw-sepia)',
            'var(--tw-drop-shadow)',
          ].join(' '),
        },
      })
      addUtilities(
        {
          '.filter': { '@defaults filter': {}, filter: 'var(--tw-filter)' },
          '.filter-none': { filter: 'none' },
        },
        variants('filter')
      )
    } else {
      addUtilities(
        {
          '.filter': {
            '--tw-blur': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-brightness': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-contrast': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-grayscale': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-hue-rotate': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-invert': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-saturate': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-sepia': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-drop-shadow': 'var(--tw-empty,/*!*/ /*!*/)',
            filter: [
              'var(--tw-blur)',
              'var(--tw-brightness)',
              'var(--tw-contrast)',
              'var(--tw-grayscale)',
              'var(--tw-hue-rotate)',
              'var(--tw-invert)',
              'var(--tw-saturate)',
              'var(--tw-sepia)',
              'var(--tw-drop-shadow)',
            ].join(' '),
          },
          '.filter-none': { filter: 'none' },
        },
        variants('filter')
      )
    }
  }
}

function blur () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        blur: (value) => {
          return {
            '--tw-blur': `blur(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('blur'),
        variants: variants('blur'),
        type: 'any',
      }
    )
  }
}

function brightness () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        brightness: (value) => {
          return {
            '--tw-brightness': `brightness(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('brightness'),
        variants: variants('brightness'),
        type: 'any',
      }
    )
  }
}

function contrast () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        contrast: (value) => {
          return {
            '--tw-contrast': `contrast(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      { values: theme('contrast'), variants: variants('contrast'), type: 'any' }
    )
  }
}




function dropShadow () {
  return function ({ config, addUtilities, theme, variants }) {
    const utilities = _.fromPairs(
      _.map(theme('dropShadow'), (value, modifier) => {
        return [
          nameClass('drop-shadow', modifier),
          {
            '--tw-drop-shadow': Array.isArray(value)
              ? value.map((v) => `drop-shadow(${v})`).join(' ')
              : `drop-shadow(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          },
        ]
      })
    )

    addUtilities(utilities, variants('dropShadow'))
  }
}

function grayscale () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        grayscale: (value) => {
          return {
            '--tw-grayscale': `grayscale(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('grayscale'),
        variants: variants('grayscale'),
        type: 'any',
      }
    )
  }
}

function hueRotate () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'hue-rotate': (value) => {
          return {
            '--tw-hue-rotate': `hue-rotate(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('hueRotate'),
        variants: variants('hueRotate'),
        type: 'any',
      }
    )
  }
}

function invert () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        invert: (value) => {
          return {
            '--tw-invert': `invert(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('invert'),
        variants: variants('invert'),
        type: 'any',
      }
    )
  }
}

function saturate () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        saturate: (value) => {
          return {
            '--tw-saturate': `saturate(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('saturate'),
        variants: variants('saturate'),
        type: 'any',
      }
    )
  }
}

function sepia () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        sepia: (value) => {
          return {
            '--tw-sepia': `sepia(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults filter': {},
                  filter: 'var(--tw-filter)',
                }
              : {}),
          }
        },
      },
      { values: theme('sepia'), variants: variants('sepia'), type: 'any' }
    )
  }
}

function backdropFilter () {
  return function ({ config, addBase, addUtilities, variants }) {
    if (config('mode') === 'jit') {
      addBase({
        '@defaults backdrop-filter': {
          '--tw-backdrop-blur': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-brightness': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-contrast': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-grayscale': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-hue-rotate': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-invert': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-opacity': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-saturate': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-sepia': 'var(--tw-empty,/*!*/ /*!*/)',
          '--tw-backdrop-filter': [
            'var(--tw-backdrop-blur)',
            'var(--tw-backdrop-brightness)',
            'var(--tw-backdrop-contrast)',
            'var(--tw-backdrop-grayscale)',
            'var(--tw-backdrop-hue-rotate)',
            'var(--tw-backdrop-invert)',
            'var(--tw-backdrop-opacity)',
            'var(--tw-backdrop-saturate)',
            'var(--tw-backdrop-sepia)',
          ].join(' '),
        },
      })
      addUtilities(
        {
          '.backdrop-filter': {
            '@defaults backdrop-filter': {},
            'backdrop-filter': 'var(--tw-backdrop-filter)',
          },
          '.backdrop-filter-none': { 'backdrop-filter': 'none' },
        },
        variants('backdropFilter')
      )
    } else {
      addUtilities(
        {
          '.backdrop-filter': {
            '--tw-backdrop-blur': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-brightness': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-contrast': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-grayscale': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-hue-rotate': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-invert': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-opacity': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-saturate': 'var(--tw-empty,/*!*/ /*!*/)',
            '--tw-backdrop-sepia': 'var(--tw-empty,/*!*/ /*!*/)',
            'backdrop-filter': [
              'var(--tw-backdrop-blur)',
              'var(--tw-backdrop-brightness)',
              'var(--tw-backdrop-contrast)',
              'var(--tw-backdrop-grayscale)',
              'var(--tw-backdrop-hue-rotate)',
              'var(--tw-backdrop-invert)',
              'var(--tw-backdrop-opacity)',
              'var(--tw-backdrop-saturate)',
              'var(--tw-backdrop-sepia)',
            ].join(' '),
          },
          '.backdrop-filter-none': { 'backdrop-filter': 'none' },
        },
        variants('backdropFilter')
      )
    }
  }
}

function backdropBlur () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-blur': (value) => {
          return {
            '--tw-backdrop-blur': `blur(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropBlur'),
        variants: variants('backdropBlur'),
        type: 'any',
      }
    )
  }
}

function backdropBrightness () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-brightness': (value) => {
          return {
            '--tw-backdrop-brightness': `brightness(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropBrightness'),
        variants: variants('backdropBrightness'),
        type: 'any',
      }
    )
  }
}

function backdropContrast () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-contrast': (value) => {
          return {
            '--tw-backdrop-contrast': `contrast(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropContrast'),
        variants: variants('backdropContrast'),
        type: 'any',
      }
    )
  }
}

function backdropGrayscale () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-grayscale': (value) => {
          return {
            '--tw-backdrop-grayscale': `grayscale(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropGrayscale'),
        variants: variants('backdropGrayscale'),
        type: 'any',
      }
    )
  }
}

function backdropHueRotate () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-hue-rotate': (value) => {
          return {
            '--tw-backdrop-hue-rotate': `hue-rotate(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropHueRotate'),
        variants: variants('backdropHueRotate'),
        type: 'any',
      }
    )
  }
}

function backdropInvert () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-invert': (value) => {
          return {
            '--tw-backdrop-invert': `invert(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropInvert'),
        variants: variants('backdropInvert'),
        type: 'any',
      }
    )
  }
}

function backdropOpacity () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-opacity': (value) => {
          return {
            '--tw-backdrop-opacity': `opacity(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropOpacity'),
        variants: variants('backdropOpacity'),
        type: 'any',
      }
    )
  }
}

function backdropSaturate () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-saturate': (value) => {
          return {
            '--tw-backdrop-saturate': `saturate(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropSaturate'),
        variants: variants('backdropSaturate'),
        type: 'any',
      }
    )
  }
}

function backdropSepia () {
  return function ({ config, matchUtilities, theme, variants }) {
    matchUtilities(
      {
        'backdrop-sepia': (value) => {
          return {
            '--tw-backdrop-sepia': `sepia(${value})`,
            ...(config('mode') === 'jit'
              ? {
                  '@defaults backdrop-filter': {},
                  'backdrop-filter': 'var(--tw-backdrop-filter)',
                }
              : {}),
          }
        },
      },
      {
        values: theme('backdropSepia'),
        variants: variants('backdropSepia'),
        type: 'any',
      }
    )
  }
}

function transitionProperty () {
  return function ({ matchUtilities, theme, variants }) {
    let defaultTimingFunction = theme('transitionTimingFunction.DEFAULT')
    let defaultDuration = theme('transitionDuration.DEFAULT')

    matchUtilities(
      {
        transition: (value) => {
          return {
            'transition-property': value,
            ...(value === 'none'
              ? {}
              : {
                  'transition-timing-function': defaultTimingFunction,
                  'transition-duration': defaultDuration,
                }),
          }
        },
      },
      {
        values: theme('transitionProperty'),
        variants: variants('transitionProperty'),
        type: 'lookup',
      }
    )
  }
}



function transitionDelay () {
  return createUtilityPlugin('transitionDelay', [['delay', ['transitionDelay']]])
}



function transitionDuration () {
  return createUtilityPlugin('transitionDuration', [['duration', ['transitionDuration']]], {
    filterDefault: true,
  })
}



function transitionTimingFunction () {
  return createUtilityPlugin('transitionTimingFunction', [['ease', ['transitionTimingFunction']]], {
    filterDefault: true,
  })
}



function content () {
  return createUtilityPlugin('content')
}
let corePluginList = [accessibility,pointerEvents,visibility,position,inset,isolation,zIndex,order,gridColumn,gridColumnStart,gridColumnEnd,gridRow,gridRowStart,gridRowEnd,float,clear,margin,boxSizing,display,height,maxHeight,minHeight,width,minWidth,maxWidth,flex,flexShrink,flexGrow,tableLayout,borderCollapse,transformOrigin,translate,rotate,skew,scale,transform,animation,cursor,userSelect,resize,listStylePosition,listStyleType,appearance,gridAutoColumns,gridAutoFlow,gridAutoRows,gridTemplateColumns,gridTemplateRows,flexDirection,flexWrap,placeContent,placeItems,alignContent,alignItems,justifyContent,justifyItems,gap,space,divideWidth,divideStyle,divideColor,divideOpacity,placeSelf,alignSelf,justifySelf,overflow,overscrollBehavior,textOverflow,whitespace,wordBreak,borderRadius,borderWidth,borderStyle,borderColor,borderOpacity,backgroundColor,backgroundOpacity,backgroundImage,gradientColorStops,boxDecorationBreak,backgroundSize,backgroundAttachment,backgroundClip,backgroundPosition,backgroundRepeat,backgroundOrigin,fill,stroke,strokeWidth,objectFit,objectPosition,padding,textAlign,verticalAlign,fontFamily,fontSize,fontWeight,textTransform,fontStyle,fontVariantNumeric,lineHeight,letterSpacing,textColor,textOpacity,textDecoration,fontSmoothing,placeholderColor,placeholderOpacity,caretColor,opacity,backgroundBlendMode,mixBlendMode,boxShadow,outline,ringWidth,ringColor,ringOpacity,ringOffsetWidth,ringOffsetColor,blur,brightness,contrast,dropShadow,grayscale,hueRotate,invert,saturate,sepia,filter,backdropBlur,backdropBrightness,backdropContrast,backdropGrayscale,backdropHueRotate,backdropInvert,backdropOpacity,backdropSaturate,backdropSepia,backdropFilter,transitionProperty,transitionDelay,transitionDuration,transitionTimingFunction,content]
function copyArray(source, array) {
    let index = -1
    const length = source.length
  
    array || (array = new Array(length))
    while (++index < length) {
      array[index] = source[index]
    }
    return array
  }

  function toKey(value) {
    if (typeof value === 'string' || isSymbol(value)) {
      return value
    }
    const result = `${value}`
    return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result
  }

  function castPath(value, object) {
    if (Array.isArray(value)) {
      return value
    }
    return isKey(value, object) ? [value] : stringToPath(value)
  }

  function baseGet(object, path) {
    path = castPath(path, object)
  
    let index = 0
    const length = path.length
  
    while (object != null && index < length) {
      object = object[toKey(path[index++])]
    }
    return (index && index == length) ? object : undefined
  }
  

  function get(object, path, defaultValue) {
    const result = object == null ? undefined : baseGet(object, path)
    return result === undefined ? defaultValue : result
  }

  function memoize(func, resolver) {
    if (typeof func !== 'function' || (resolver != null && typeof resolver !== 'function')) {
      throw new TypeError('Expected a function')
    }
    const memoized = function(...args) {
      const key = resolver ? resolver.apply(this, args) : args[0]
      const cache = memoized.cache
  
      if (cache.has(key)) {
        return cache.get(key)
      }
      const result = func.apply(this, args)
      memoized.cache = cache.set(key, result) || cache
      return result
    }
    memoized.cache = new (memoize.Cache || Map)
    return memoized
  }
  
  memoize.Cache = Map

  const MAX_MEMOIZE_SIZE = 500

  function memoizeCapped(func) {
    const result = memoize(func, (key) => {
      const { cache } = result
      if (cache.size === MAX_MEMOIZE_SIZE) {
        cache.clear()
      }
      return key
    })
  
    return result
  }

function isSymbol(value) {
    const type = typeof value
    return type == 'symbol' || (type === 'object' && value != null && getTag(value) == '[object Symbol]')
  }

const reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/
const reIsPlainProp = /^\w*$/

function isFunction(value) {
    return typeof value === 'function'
  }
  
function isKey(value, object) {
  if (Array.isArray(value)) {
    return false
  }
  const type = typeof value
  if (type === 'number' || type === 'boolean' || value == null || isSymbol(value)) {
    return true
  }
  return reIsPlainProp.test(value) || !reIsDeepProp.test(value) ||
    (object != null && value in Object(object))
}

function keysIn(object) {
    const result = []
    for (const key in object) {
      result.push(key)
    }
    return result
  }

function baseMerge(object, source, srcIndex, customizer, stack) {
    if (object === source) {
      return
    }
    baseFor(source, (srcValue, key) => {
      if (isObject(srcValue)) {
        stack || (stack = new Stack)
        baseMergeDeep(object, source, key, srcIndex, baseMerge, customizer, stack)
      }
      else {
        let newValue = customizer
          ? customizer(object[key], srcValue, `${key}`, object, source, stack)
          : undefined
  
        if (newValue === undefined) {
          newValue = srcValue
        }
        assignMergeValue(object, key, newValue)
      }
    }, keysIn)
  }

  function baseAssignValue(object, key, value) {
    if (key == '__proto__') {
      Object.defineProperty(object, key, {
        'configurable': true,
        'enumerable': true,
        'value': value,
        'writable': true
      })
    } else {
      object[key] = value
    }
  }

  function assignMergeValue(object, key, value) {
    if ((value !== undefined && !eq(object[key], value)) ||
        (value === undefined && !(key in object))) {
      baseAssignValue(object, key, value)
    }
  }

  function baseMergeDeep(object, source, key, srcIndex, mergeFunc, customizer, stack) {
    const objValue = object[key]
    const srcValue = source[key]
    const stacked = stack.get(srcValue)
  
    if (stacked) {
      assignMergeValue(object, key, stacked)
      return
    }
    let newValue = customizer
      ? customizer(objValue, srcValue, `${key}`, object, source, stack)
      : undefined
  
    let isCommon = newValue === undefined
  
    if (isCommon) {
      const isArr = Array.isArray(srcValue)
      const isBuff = !isArr && isBuffer(srcValue)
      const isTyped = !isArr && !isBuff && isTypedArray(srcValue)
  
      newValue = srcValue
      if (isArr || isBuff || isTyped) {
        if (Array.isArray(objValue)) {
          newValue = objValue
        }
        else if (isArrayLikeObject(objValue)) {
          newValue = copyArray(objValue)
        }
        else if (isBuff) {
          isCommon = false
          newValue = cloneBuffer(srcValue, true)
        }
        else if (isTyped) {
          isCommon = false
          newValue = cloneTypedArray(srcValue, true)
        }
        else {
          newValue = []
        }
      }
      else if (isPlainObject(srcValue) || isArguments(srcValue)) {
        newValue = objValue
        if (isArguments(objValue)) {
          newValue = toPlainObject(objValue)
        }
        else if (typeof objValue === 'function' || !isObject(objValue)) {
          newValue = initCloneObject(srcValue)
        }
      }
      else {
        isCommon = false
      }
    }
    if (isCommon) {
      // Recursively merge objects and arrays (susceptible to call stack limits).
      stack.set(srcValue, newValue)
      mergeFunc(newValue, srcValue, srcIndex, customizer, stack)
      stack['delete'](srcValue)
    }
    assignMergeValue(object, key, newValue)
  }


  function isArrayLike(value) {
    return value != null && typeof value !== 'function' && isLength(value.length)
  }

  const MAX_SAFE_INTEGER = 9007199254740991

const reIsUint = /^(?:0|[1-9]\d*)$/

function isIndex(value, length) {
  const type = typeof value
  length = length == null ? MAX_SAFE_INTEGER : length

  return !!length &&
    (type === 'number' ||
      (type !== 'symbol' && reIsUint.test(value))) &&
        (value > -1 && value % 1 == 0 && value < length)
}


  function isIterateeCall(value, index, object) {
    if (!isObject(object)) {
      return false
    }
    const type = typeof index
    if (type === 'number'
      ? (isArrayLike(object) && isIndex(index, object.length))
      : (type === 'string' && index in object)
    ) {
      return eq(object[index], value)
    }
    return false
  }
  
  
  function createAssigner(assigner) {
    return (object, ...sources) => {
      let index = -1
      let length = sources.length
      let customizer = length > 1 ? sources[length - 1] : undefined
      const guard = length > 2 ? sources[2] : undefined
  
      customizer = (assigner.length > 3 && typeof customizer === 'function')
        ? (length--, customizer)
        : undefined
  
      if (guard && isIterateeCall(sources[0], sources[1], guard)) {
        customizer = length < 3 ? undefined : customizer
        length = 1
      }
      object = Object(object)
      while (++index < length) {
        const source = sources[index]
        if (source) {
          assigner(object, source, index, customizer)
        }
      }
      return object
    }
  }

const mergeWith = createAssigner((object, source, srcIndex, customizer) => {
    baseMerge(object, source, srcIndex, customizer)
  })// import some from 'lodash/some'
// import mergeWith from 'lodash/mergeWith'
// import isFunction from 'lodash/isFunction'
// import isUndefined from 'lodash/isUndefined'
// import defaults from 'lodash/defaults'
// import map from 'lodash/map'
// import get from 'lodash/get'
// import uniq from 'lodash/uniq'
// import toPath from 'lodash/toPath'
// import head from 'lodash/head'
// import isPlainObject from 'lodash/isPlainObject'
// import negateValue from './negateValue'


// import configurePlugins from './configurePlugins'
//import corePluginList from '../corePluginList'
//import defaultConfig from '../../stubs/defaultConfig.stub'
//import colors from '../../colors'

const configUtils = {
  colors,
  negative(scale) {
    return Object.keys(scale)
      .filter((key) => scale[key] !== '0')
      .reduce(
        (negativeScale, key) => ({
          ...negativeScale,
          [`-${key}`]: negateValue(scale[key]),
        }),
        {}
      )
  },
  breakpoints(screens) {
    return Object.keys(screens)
      .filter((key) => typeof screens[key] === 'string')
      .reduce(
        (breakpoints, key) => ({
          ...breakpoints,
          [`screen-${key}`]: screens[key],
        }),
        {}
      )
  },
}

function value(valueToResolve, ...args) {
  return isFunction(valueToResolve) ? valueToResolve(...args) : valueToResolve
}

function collectExtends(items) {
  return items.reduce((merged, { extend }) => {
    return mergeWith(merged, extend, (mergedValue, extendValue) => {
      if (isUndefined(mergedValue)) {
        return [extendValue]
      }

      if (Array.isArray(mergedValue)) {
        return [extendValue, ...mergedValue]
      }

      return [extendValue, mergedValue]
    })
  }, {})
}

function mergeThemes(themes) {
  return {
    ...themes.reduce((merged, theme) => defaults(merged, theme), {}),

    // In order to resolve n config objects, we combine all of their `extend` properties
    // into arrays instead of objects so they aren't overridden.
    extend: collectExtends(themes),
  }
}

function mergeExtensionCustomizer(merged, value) {
  // When we have an array of objects, we do want to merge it
  if (Array.isArray(merged) && isPlainObject(head(merged))) {
    return merged.concat(value)
  }

  // When the incoming value is an array, and the existing config is an object, prepend the existing object
  if (Array.isArray(value) && isPlainObject(head(value)) && isPlainObject(merged)) {
    return [merged, ...value]
  }

  // Override arrays (for example for font-families, box-shadows, ...)
  if (Array.isArray(value)) {
    return value
  }

  // Execute default behaviour
  return undefined
}

function mergeExtensions({ extend, ...theme }) {
  return mergeWith(theme, extend, (themeValue, extensions) => {
    // The `extend` property is an array, so we need to check if it contains any functions
    if (!isFunction(themeValue) && !some(extensions, isFunction)) {
      return mergeWith({}, themeValue, ...extensions, mergeExtensionCustomizer)
    }

    return (resolveThemePath, utils) =>
      mergeWith(
        {},
        ...[themeValue, ...extensions].map((e) => value(e, resolveThemePath, utils)),
        mergeExtensionCustomizer
      )
  })
}

function resolveFunctionKeys(object) {
  const resolvePath = (key, defaultValue) => {
    const path = toPath(key)

    let index = 0
    let val = object

    while (val !== undefined && val !== null && index < path.length) {
      val = val[path[index++]]
      val = isFunction(val) ? val(resolvePath, configUtils) : val
    }

    return val === undefined ? defaultValue : val
  }

  return Object.keys(object).reduce((resolved, key) => {
    return {
      ...resolved,
      [key]: isFunction(object[key]) ? object[key](resolvePath, configUtils) : object[key],
    }
  }, {})
}

function extractPluginConfigs(configs) {
  let allConfigs = []

  configs.forEach((config) => {
    allConfigs = [...allConfigs, config]

    const plugins = get(config, 'plugins', [])

    if (plugins.length === 0) {
      return
    }

    plugins.forEach((plugin) => {
      if (plugin.__isOptionsFunction) {
        plugin = plugin()
      }
      allConfigs = [...allConfigs, ...extractPluginConfigs([get(plugin, 'config', {})])]
    })
  })

  return allConfigs
}

function mergeVariants(variants) {
  const mergedVariants = variants.reduce((resolved, variants) => {
    Object.entries(variants || {}).forEach(([plugin, pluginVariants]) => {
      if (isFunction(pluginVariants)) {
        resolved[plugin] = pluginVariants({
          variants(path) {
            return get(resolved, path, [])
          },
          before(toInsert, variant, existingPluginVariants = get(resolved, plugin, [])) {
            if (variant === undefined) {
              return [...toInsert, ...existingPluginVariants]
            }

            const index = existingPluginVariants.indexOf(variant)

            if (index === -1) {
              return [...existingPluginVariants, ...toInsert]
            }

            return [
              ...existingPluginVariants.slice(0, index),
              ...toInsert,
              ...existingPluginVariants.slice(index),
            ]
          },
          after(toInsert, variant, existingPluginVariants = get(resolved, plugin, [])) {
            if (variant === undefined) {
              return [...existingPluginVariants, ...toInsert]
            }

            const index = existingPluginVariants.indexOf(variant)

            if (index === -1) {
              return [...toInsert, ...existingPluginVariants]
            }

            return [
              ...existingPluginVariants.slice(0, index + 1),
              ...toInsert,
              ...existingPluginVariants.slice(index + 1),
            ]
          },
          without(toRemove, existingPluginVariants = get(resolved, plugin, [])) {
            return existingPluginVariants.filter((v) => !toRemove.includes(v))
          },
        })
      } else {
        resolved[plugin] = pluginVariants
      }
    })

    return resolved
  }, {})

  return {
    ...mergedVariants,
    extend: collectExtends(variants),
  }
}

function mergeVariantExtensions({ extend, ...variants }, variantOrder) {
  return mergeWith(variants, extend, (variantsValue, extensions) => {
    const merged = uniq([...(variantsValue || []), ...extensions].flat())

    if (extensions.flat().length === 0) {
      return merged
    }

    return merged.sort((a, z) => variantOrder.indexOf(a) - variantOrder.indexOf(z))
  })
}

function resolveVariants([firstConfig, ...variantConfigs], variantOrder) {
  // Global variants configuration like `variants: ['hover', 'focus']`
  if (Array.isArray(firstConfig)) {
    return firstConfig
  }

  return mergeVariantExtensions(
    mergeVariants([firstConfig, ...variantConfigs].reverse()),
    variantOrder
  )
}

function resolveCorePlugins(corePluginConfigs) {
  const result = [...corePluginConfigs].reduceRight((resolved, corePluginConfig) => {
    if (isFunction(corePluginConfig)) {
      return corePluginConfig({ corePlugins: resolved })
    }
    return configurePlugins(corePluginConfig, resolved)
  }, corePluginList)

  return result
}

function resolvePluginLists(pluginLists) {
  const result = [...pluginLists].reduceRight((resolved, pluginList) => {
    return [...resolved, ...pluginList]
  }, [])

  return result
}

function resolveConfigObjects(configs) {
  const allConfigs = [
    ...extractPluginConfigs(configs),
    {
      darkMode: false,
      prefix: '',
      important: false,
      separator: ':',
      variantOrder: defaultConfig.variantOrder,
    },
  ]
  const { variantOrder } = allConfigs.find((c) => c.variantOrder)

  return defaults(
    {
      theme: resolveFunctionKeys(
        mergeExtensions(mergeThemes(map(allConfigs, (t) => get(t, 'theme', {}))))
      ),
      variants: resolveVariants(
        allConfigs.map((c) => get(c, 'variants', {})),
        variantOrder
      ),
      corePlugins: resolveCorePlugins(allConfigs.map((c) => c.corePlugins)),
      plugins: resolvePluginLists(configs.map((c) => get(c, 'plugins', []))),
    },
    ...allConfigs
  )
}

//import { flagEnabled } from '../featureFlags'
let flagEnabled = () => false;

function getAllConfigs(config) {
  const configs = (config?.presets ?? [defaultConfig])
    .slice()
    .reverse()
    .flatMap((preset) => getAllConfigs(preset instanceof Function ? preset() : preset))

  const features = {
    // Add experimental configs here...
  }

  const experimentals = Object.keys(features)
    .filter((feature) => flagEnabled(config, feature))
    .map((feature) => features[feature])

  return [config, ...experimentals, ...configs]
}

function resolveConfigInternal(...configs) {
  const [, ...defaultConfigs] = getAllConfigs(configs[0])
  return resolveConfigObjects([...configs, ...defaultConfigs])
}

function resolveConfig() {
    let config = {}; // configPath ? require(configPath) : 
    let resolvedConfig = resolveConfigInternal(config)

    // if (args['--purge']) {
    //   resolvedConfig.purge = {
    //     enabled: true,
    //     content: args['--purge'].split(/(?<!{[^}]+),/),
    //   }
    // }

    // if (args['--jit']) {
    //   resolvedConfig.mode = 'jit'
    // }

    return resolvedConfig
  }

let tailwindResult = [];
let tailwindResultBase = [];

let getDeepValue = (path, obj, defaultValue) => {
    let result = null;
    let parts = path.split(".");
    for (var p of parts) {
        result = obj[p];
        if (!result) {
            return defaultValue;
        }
    }
    return result;
} 

let resolvedConfig = resolveConfig()


const getConfigValue = (path, defaultValue) => (path ? get(resolvedConfig, path, defaultValue) : defaultConfig)


function main() {
let context = {
    instance: (plugin) => {
        context.currentPlugin = plugin;
        return context;
    },    
    prefix: (str) => str,
    addBase: (obj) => tailwindResultBase.push(obj), 
    corePlugins: (key) => true,
    theme: (path, defaultValue) => {
        const [pathRoot, ...subPaths] = toPath(path)
        const value = getConfigValue(['theme', pathRoot, ...subPaths], defaultValue)
        return transformThemeValue(pathRoot)(value)
      },
    theme2: (path,defaultValue) => getDeepValue(path, resolvedConfig.theme),
    config: (key) => (key == 'mode') ? false : null,
    variants: (path, defaultValue) => getDeepValue(path, resolvedConfig.variants),
    addUtilities3: addUtilities,
    addUtilities: addUtilities2, 
    addComponents: (components, options) => tailwindResultBase.push({components, options}),
    matchUtilities: (matches, { values, variants, respectPrefix, respectImportant }) => {
        let modifierValues = Object.entries(values)
        let result = Object.entries(matches).flatMap(([name, utilityFunction]) => {
          return modifierValues
            .map(([modifier, value]) => {
              let declarations = utilityFunction(value, {
                includeRules(rules, options) {
                    context.addUtilities(rules, options, context.currentPlugin)
                },
              })
              if (!declarations) {
                return null
              }
              return {
                [nameClass(name, modifier)]: declarations,
              }
            })
            .filter(Boolean)
        })
        context.addUtilities(result, { variants, respectPrefix, respectImportant }, context.currentPlugin)
    }
};
let p = {container,accessibility,pointerEvents,visibility,position,inset,isolation,zIndex,order,gridColumn,gridColumnStart,gridColumnEnd,gridRow,gridRowStart,gridRowEnd,float,clear,margin,boxSizing,display,height,maxHeight,minHeight,width,minWidth,maxWidth,flex,flexShrink,flexGrow,tableLayout,borderCollapse,transformOrigin,transform,translate,rotate,skew,scale,animation,cursor,userSelect,resize,listStylePosition,listStyleType,appearance,gridAutoColumns,gridAutoFlow,gridAutoRows,gridTemplateColumns,gridTemplateRows,flexDirection,flexWrap,placeContent,placeItems,alignContent,alignItems,justifyContent,justifyItems,gap,space,divideWidth,divideStyle,divideColor,divideOpacity,placeSelf,alignSelf,justifySelf,overflow,overscrollBehavior,textOverflow,whitespace,wordBreak,borderRadius,borderWidth,borderStyle,borderColor,borderOpacity,backgroundColor,backgroundOpacity,backgroundImage,gradientColorStops,boxDecorationBreak,backgroundSize,backgroundAttachment,backgroundClip,backgroundPosition,backgroundRepeat,backgroundOrigin,fill,stroke,strokeWidth,objectFit,objectPosition,padding,textAlign,verticalAlign,fontFamily,fontSize,fontWeight,textTransform,fontStyle,fontVariantNumeric,lineHeight,letterSpacing,textColor,textOpacity,textDecoration,fontSmoothing,placeholderColor,placeholderOpacity,caretColor,opacity,backgroundBlendMode,mixBlendMode,boxShadow,outline,ringWidth,ringColor,ringOpacity,ringOffsetWidth,ringOffsetColor,filter,blur,brightness,contrast,dropShadow,grayscale,hueRotate,invert,saturate,sepia,backdropFilter,backdropBlur,backdropBrightness,backdropContrast,backdropGrayscale,backdropHueRotate,backdropInvert,backdropOpacity,backdropSaturate,backdropSepia,transitionProperty,transitionDelay,transitionDuration,transitionTimingFunction,content};


// let f = zIndex;
// let instace = f();
// instace(context);


for (var t=0; t<myPluginOrder.length; t++) {
       let f = myPluginOrder[t];
       let instace = eval(f+"()");
    //    let instace = f();
       instace(context.instance(f));
}

//for (var key in p) {
    //    let f = p[key];
    //let instace = f();
    //instace(context);
    //}

let test = tailwindResult.map((tr, index) => generateCss(index, tr)).join("");


test = modernNormalizeCss + preflightCss + test;

require('fs').writeFileSync("./output/test.css", test);

}    
main();
// console.log(tailwindResult);
